<?php

	/*=======================================================================
	 * copyright (c) AL-Design Works 
	 * http://cms.al-design.jp/
	 * このソースファイルは CMS Designer の一部です。
	 * 詳しいライセンス規約についてはウェブサイトを参照してください。
	 *=======================================================================
	 */
	
	if ( version_compare(PHP_VERSION,'5','>=') )
	{
 		require_once('domxml-php4-to-php5.php');
 	}
 	
	if ( version_compare(PHP_VERSION,'5','>=') && extension_loaded('xsl') )
	{
		require_once('xslt-php4-to-php5.php');
	}
	
	/* CMSD設定ファイルの読み込み */
	require_once( 'cmsdconfig.php.inc' );
	
	/* -----------------------------------------------------------
	 * [共通関数]
	 * HTML出力用にUTF-8をEUC-JPに変換する。→2015.08.27改定。内部文字コードをEUC-JPからUTF-8に統一した為、基本的には何も処理しない関数となった。
	 */
	function convencoding_xml2html( $str )
	{
		//$str = utf8_replace_irregular_chars( $str );
		if ( gCMSDEnableConvertingEncoding() ) {
			$str = htmlentities( 
				mb_convert_encoding( $str, gCMSDInternalEncodingPHP(), 'UTF-8' ), 
				ENT_QUOTES, gCMSDInternalEncoding()
			);
		}
		return $str;
	}
	
	/* -----------------------------------------------------------
	 * [共通関数]
	 * xmlデータをCMSD内部文字コードに変換する際の関数。→2015.08.27改定。内部文字コードをEUC-JPからUTF-8に統一した為、基本的には何も処理しない関数となった。
	 */
	function file2code( $str )
	{
		if ( gCMSDEnableConvertingEncoding() ) {
			//$str = utf8_replace_irregular_chars( $str );
			$str = mb_convert_encoding( $str, gCMSDInternalEncodingPHP(), 'UTF-8' );
		}
		return $str;
	}
	
	/* -----------------------------------------------------------
	 * [共通関数]
	 * CMSD内部文字コードをxmlに保存する際の関数。→2015.08.27改定。内部文字コードをEUC-JPからUTF-8に統一した為、基本的には何も処理しない関数となった。
	 */
	function fileconv( $str )
	{
		if ( gCMSDEnableConvertingEncoding() ) {
			$str = htmlspecialchars( mb_convert_encoding( $str, 'UTF-8', gCMSDInternalEncodingPHP() ), ENT_QUOTES, 'UTF-8' );//mb_detect_encoding( $str ) ) );
			//$str = utf8_replace_irregular_chars( $str ); // 2007.10.1 追加。eucJP-win対応による副作用を回避。//→XSLTをかける側で対処するようにした。
			
			// ↓IBM拡張文字の変換がうまくできなかったので以下の方法を試したが、今度は小さい丸文字の(R)が変換できなくなった。
			//$str = htmlspecialchars( mb_convert_encoding( mb_convert_encoding( $str, 'sjis-win', 'EUC-JP' ), 'UTF-8', 'sjis-win' ), ENT_QUOTES, 'UTF-8' );//mb_detect_encoding( $str ) ) );
		}
		return $str;
	}
	
	function cmsd_session_start()
	{
		// セッションクッキーをHttpOnlyにする
		
		if ( version_compare(PHP_VERSION,'5.2.0','>=') )
		{
			$currentCookieParams = session_get_cookie_params();
			session_set_cookie_params( 
				$currentCookieParams["lifetime"], 
				$currentCookieParams["path"], 
				$currentCookieParams["domain"], 
				$currentCookieParams["secure"], 
				true
			);
		} else {
			header( "Set-Cookie: name=value; httpOnly" );
		}
		
		session_name( 'cmsdsessionid' );
		session_start();
	}
	
	// GETパラメータを取得する。
	function getGETParam( $paramName, $defaultValue = null )
	{
		$value = $defaultValue;
		if ( isset( $_GET[ $paramName ] ) )
		{
			$value = $_GET[ $paramName ];
		}
		
		return $value;
	}
	
	// POSTパラメータを取得する。
	function getPOSTParam( $paramName, $defaultValue = null )
	{
		$value = $defaultValue;
		if ( isset( $_POST[ $paramName ] ) )
		{
			$value = $_POST[ $paramName ];
		}
		
		return $value;
	}
	
	
	// デフォルト値付き値取得
	function nullString( $value, $defaultValue = "" )
	{
		if ( $value === '' or is_null( $value ) )
		{
			return $defaultValue;
		}
		else
		{
			return $value;
		}
	}

	 // ブーリアン型文字列（'True'|'False'）をパースする。
	 function parseBoolean( $strValue, $default = false )
	 {
	 	$value = $default;
	 
	 	if ( strcasecmp( $strValue, 'TRUE' ) == 0 )
		{
			$value = true;
		}
		elseif ( strcasecmp( $strValue, 'FALSE' ) == 0 )
		{
			$value = false;
		}
		
		return $value;
	 
	 }
	 
	 // ファイル等から文字列値を取得する（規定値付き）
	 function parseString( $strValue, $default = '' )
	 {
	 	$value = $default;
	 	
		if ( strlen( $strValue ) > 0 )
		{
			$value = $strValue;
		}
	 	
		return $value;
	 }
	 
	 function parseNumeric( $strValue, $default = 0 )
	 {
	 	$value = $default;
		
		if ( is_numeric( $strValue) )
		{
			$value = $strValue;
		}
		
		return $value;
	 }

	/* -----------------------------------------------------------
	 * ファイルサイズをバイト数から単位付の数値に変換する。
	 *	
	 */
	function byte2unit( $fileSize )
	{
		$unitFileSize = 0; // 初期化
		// ファイルサイズ上限
		if ( $fileSize > 0 and $fileSize < 1024 )
		{
			$unitFileSize =  $fileSize . "bytes";
		}
		else if ( $fileSize >= 1024 and $fileSize < 1024000 )
		{
			$unitFileSize = round( $fileSize / 1024, 1 ) . "KB";
		}
		else if ( $fileSize >= 1024000 )
		{
			$unitFileSize =  round( $fileSize / 1024000, 1 ) . "MB";
		}
		
		return $unitFileSize;
	
	}
	
	function unit2byte( $unitFileSize )
	{
		$fileSize = 0; // 初期化
		if ( !empty($unitFileSize) && is_numeric( $unitFileSize ) )
		{
			$fileSize = $unitFileSize;
		}
		else if ( !empty($unitFileSize) && !is_numeric( $unitFileSize ) )
		{
			// 単位を切り出す（後ろ2文字）
			$unitkind = substr( $unitFileSize, -2 );
			
			// 値を切り出す（後ろ2文字を除いた値）
			$sizevalue = substr( $unitFileSize, 0, strlen( $unitFileSize ) - 2 );
			
			if ( !empty($sizevalue) && is_numeric( $sizevalue ) )
			{
				// 'KB'の場合は1024倍する。
				switch( strtoupper( $unitkind ) )
				{
					case 'KB' :
						$fileSize = $sizevalue * 1024;
						break;
				
					case 'MB' :
						$fileSize = $sizevalue * 1024 * 1000;
						break;
				}
				
			}
		}
		
		return $fileSize;
	}
	
	
	/* -----------------------------------------------------------
	 * die関数のラッパー。XSS対応。
	 *	
	 */
	function cmsd_abort_proc( $message, $sanitizing = true )
	{
		if ( $sanitizing )
		{
			$message = htmlspecialchars( $message, ENT_QUOTES );
		}
		$html = '<html><head><title>CMS Designer</title><meta http-equiv="Content-Type" content="text/html; charset=' . gCMSDInternalEncoding() . '" /></head><body>' . $message . '</body></html>';
		die($html );
	}
	
	/* -----------------------------------------------------------
	 * PHP4では objがnullならobj->propもnullになっていたが、PHP5からはNoticeが出るようになった為、
	 * この挙動を前提にしていた foreach( xml->nodeset as hoge ) の部分を簡単に修正できるようにする。
	 */
	function cmsd_get_nodeset( $xmlobj )
	{
		if ( is_object( $xmlobj ) )
		{
			return $xmlobj->nodeset;
		}
		else
		{
			return array();
		}
	}

	/* ---------------------------------------------------------------
	 * アプリケーションクラス。
	 */
	class App
	{
		/* -----------------------------------------------------------
		 * [クラスメソッド]
		 * CMSDesignerのルートディレクトリパスの取得。
		 */
		public static function getRootDir()
		{
			return rtrim( realpath( dirname(__FILE__) . "/../" ), '/\\');	// 2007.11.21 PHP5でrealpath関数の戻り値の末尾に/や\が付く仕様変更に対応。
		}
		
		/* -----------------------------------------------------------
		 * [クラスメソッド]
		 * CMSDesignerのルートディレクトリ名の取得。
		 */
		public static function getRootDirName()
		{
			// ルートディレクトリパス文字列を"/"で区切って最後の要素を取り出せば、それがDir名なはず。
			$rootDirPath = App::getRootDir();
			$tokens = preg_split( '@[\\\/]@', $rootDirPath );	// LinuxとWindowsどちらでも対応できるように/と\を両方デリミタとしている。
			return array_pop( $tokens );
		}
		
		/* -----------------------------------------------------------
		 * [クラスメソッド]
		 * キャッシュディレクトリパスの取得。
		 */
		public static function getCacheDir()
		{
			return App::getRootDir() . '/cache/';
		}
		
		/* -----------------------------------------------------------
		 * [クラスメソッド]
		 * エントリのディレクトリパスの取得。
		 */
		public static function createEntryDirPath( $entryName )
		{
			if ( !InputValidater::IsEntryName( $entryName ) )
			{
				cmsd_abort_proc( 'invalid entry name : ' . $entryName );
			}
			
			$dirpath = App::getRootDir() . "/data/entry/" . $entryName . "/";

			// dirpathの存在チェック
			if ( ! file_exists( $dirpath )	)
			{
				cmsd_abort_proc( "エントリフォルダ'$dirpath'がありません。パーミッション705以上でフォルダを作成してください。" );
			}
			
			return $dirpath;
		}
	
		/* -----------------------------------------------------------
		 * [クラスメソッド]
		 * エントリファイル名の取得。
		 */
		public static function createEntryFileName( $entryName, $id )
		{
			if ( !InputValidater::IsEntryName( $entryName ) )
			{
				cmsd_abort_proc( 'invalid entry name : ' . $entryName );
			}
			
			if ( !InputValidater::IsEntryId( $id ) )
			{
				cmsd_abort_proc( 'invalid entry ID : ' . $id );
			}
			
			return $entryName . "." . $id . ".xml";
		}
	
		/* -----------------------------------------------------------
		 * [クラスメソッド]
		 * エントリファイルパスの取得。
		 */
		public static function createEntryFilePath( $entryName, $id )
		{
			return App::createEntryDirPath( $entryName ) . App::createEntryFileName( $entryName, $id );
		}
		
		
		public static function createEntrySequenceFilePath( $entryName )
		{
			if ( !InputValidater::IsEntryName( $entryName ) )
			{
				cmsd_abort_proc( 'invalid entry name : ' . $entryName );
			}
			
			return App::createEntryDirPath( $entryName ) . $entryName . '.sequence';
		}		
		
		/* -----------------------------------------------------------
		 * [クラスメソッド]
		 * site.config.xml ファイルパスの取得。
		 */
		public static function getSiteConfigFilePath()
		{
			return App::getRootDir() . "/config/site.config.xml";
		}

		/* -----------------------------------------------------------
		 * [クラスメソッド]
		 * users.config.xml ファイルパスの取得。
		 */
		public static function getUsersConfigFilePath()
		{
			return App::getRootDir() . "/config/users.config.xml";
		}
		
		
		/* -----------------------------------------------------------
		 * [クラスメソッド]
		 * スキーマファイルパスの取得。
		 */
		public static function createSchemaFilePath( $schemaName )
		{
			if ( !InputValidater::IsSchemaName( $schemaName ) )
			{
				cmsd_abort_proc( 'invalid schema name:' . $schemaName );
			}
			
			return App::getRootDir() . "/config/schema/" . $schemaName . "/" . $schemaName . '.schema.xml';
		}
		
		/* -----------------------------------------------------------
		 * [クラスメソッド]
		 * エントリ内ファイル名の取得。
		 */
		public static function createAttachedFileName( $entryName, $entryId, $fileId, $orgFileName )
		{
			if ( !InputValidater::IsEntryName( $entryName ) )
			{
				cmsd_abort_proc( 'invalid entry name : ' . $entryName );
			}
			
			if ( !InputValidater::IsEntryId( $entryId ) )
			{
				cmsd_abort_proc( 'invalid entry ID : ' . $entryId );
			}
			
			if ( !InputValidater::IsAttachedFileId( $fileId ) )
			{
				cmsd_abort_proc( 'invalid attached file ID : ' . $fileId );
			}
						
			return $entryName . '.' . $entryId . '.' . $fileId . '.' . getfileext( $orgFileName );
		}
		
		/* -----------------------------------------------------------
		 * [クラスメソッド]
		 * エントリ内ファイルパスの取得。
		 */
		public static function createAttachedFilePath( $entryName, $entryId, $fileId, $orgFileName )
		{
			if ( !InputValidater::IsEntryName( $entryName ) )
			{
				cmsd_abort_proc( 'invalid entry name : ' . $entryName );
			}
			
			if ( !InputValidater::IsEntryId( $entryId ) )
			{
				cmsd_abort_proc( 'invalid entry ID : ' . $entryId );
			}
			
			if ( !InputValidater::IsAttachedFileId( $fileId ) )
			{
				cmsd_abort_proc( 'invalid attached file ID : ' . $fileId );
			}
			
			return App::createEntryDirPath( $entryName ) . $entryName . '.' . $entryId . '.' . $fileId . '.' . getfileext( $orgFileName );
		}
		
		/* -----------------------------------------------------------
		 * [クラスメソッド]
		 * エントリ内ファイル名の取得。
		 */
		public static function createImageCacheFileName( $entryName, $entryId, $fileId, $width, $height, $orgFileName )
		{
			if ( !InputValidater::IsEntryName( $entryName ) )
			{
				cmsd_abort_proc( 'invalid entry name : ' . $entryName );
			}
			
			if ( !InputValidater::IsEntryId( $entryId ) )
			{
				cmsd_abort_proc( 'invalid entry ID : ' . $entryId );
			}
			
			if ( !InputValidater::IsAttachedFileId( $fileId ) )
			{
				cmsd_abort_proc( 'invalid attached file ID : ' . $fileId );
			}
			
			$width = parseNumeric( $width, 0 );
			$height = parseNumeric( $height, 0 );
			
			return $entryName . '.' . $entryId . '.' . $fileId . '.' . "{$width}x{$height}" . '.' . getfileext( $orgFileName );
		}
		
		/* -----------------------------------------------------------
		 * [クラスメソッド]
		 * エントリ内ファイルパスの取得。
		 */
		public static function createImageCacheFilePath( $entryName, $entryId, $fileId, $width, $height, $orgFileName )
		{
			return	App::getRootDir() . "/cache/imagecache." . App::createImageCacheFileName( $entryName, $entryId, $fileId, $width, $height, $orgFileName );
		}
		
		/* -----------------------------------------------------------
		 * [クラスメソッド]
		 * Xslファイル名の取得。
		 */
		public static function createXslFilePath( $schemaName, $designName )
		{
			if ( !InputValidater::IsSchemaName( $schemaName ) )
			{
				cmsd_abort_proc( 'invalid schema name : ' . $schemaName );
			}
			
			if ( !InputValidater::IsDesignName( $designName ) )
			{
				cmsd_abort_proc( 'invalid design name : ' . $designName );
			}
			
			return App::getRootDir() . "/config/schema/" . $schemaName . "/" . App::createXslFileName( $schemaName, $designName );
		}
		
		/* -----------------------------------------------------------
		 * [クラスメソッド]
		 * Xslファイルパスの取得。
		 */
		public static function createXslFileName( $schemaName, $designName )
		{
			if ( !InputValidater::IsSchemaName( $schemaName ) )
			{
				cmsd_abort_proc( 'invalid schema name : ' . $schemaName );
			}
			
			if ( !InputValidater::IsDesignName( $designName ) )
			{
				cmsd_abort_proc( 'invalid design name : ' . $designName );
			}
			
			return $schemaName . '.' . $designName . '.design.xsl';
		}
				
		/* -----------------------------------------------------------
		 * [クラスメソッド]
		 * Xslファイル名の取得。
		 */
		public static function createListXslFilePath( $schemaName, $designName )
		{
			if ( !InputValidater::IsSchemaName( $schemaName ) )
			{
				cmsd_abort_proc( 'invalid schema name : ' . $schemaName );
			}
			
			if ( !InputValidater::IsDesignName( $designName ) )
			{
				cmsd_abort_proc( 'invalid design name : ' . $designName );
			}
			
			return App::getRootDir() . "/config/schema/" . $schemaName . "/" . App::createListXslFileName( $schemaName, $designName );
		}
		
		/* -----------------------------------------------------------
		 * [クラスメソッド]
		 * Xslファイルパスの取得。
		 */
		public static function createListXslFileName( $schemaName, $designName )
		{
			if ( !InputValidater::IsSchemaName( $schemaName ) )
			{
				cmsd_abort_proc( 'invalid schema name : ' . $schemaName );
			}
			
			if ( !InputValidater::IsDesignName( $designName ) )
			{
				cmsd_abort_proc( 'invalid design name : ' . $designName );
			}

			return $schemaName . '.list.' . $designName . '.design.xsl';
		}
		
		/* -----------------------------------------------------------
		 * [クラスメソッド]
		 * エントリロックファイルパスの取得。
		 */
		public static function createEntryLockDirPath()
		{
		
			return App::getRootDir() . '/data/lock/'; 
		}		
		

		/* -----------------------------------------------------------
		 * [クラスメソッド]
		 * エントリロックファイルパスの取得。
		 */
		public static function createEntryLockFilePath( $entryName, $entryId, $userId )
		{
			if ( !InputValidater::IsEntryName( $entryName ) )
			{
				cmsd_abort_proc( 'invalid entry name : ' . $entryName );
			}
			
			if ( !InputValidater::IsEntryId( $entryId ) )
			{
				cmsd_abort_proc( 'invalid entry ID : ' . $entryId );
			}
			
			return App::createEntryLockDirPath() . $entryName . '.' . $entryId . '.' . $userId . '.lock';
		}
		
		
		public static function createImageUrl( $entryName, $entryId, $fileId, $orgFileName )
		{
			if ( !InputValidater::IsEntryName( $entryName ) )
			{
				cmsd_abort_proc( 'invalid entry name : ' . $entryName );
			}
			
			if ( !InputValidater::IsEntryId( $entryId ) )
			{
				cmsd_abort_proc( 'invalid entry ID : ' . $entryId );
			}
			
			if ( !InputValidater::IsAttachedFileId( $fileId ) )
			{
				cmsd_abort_proc( 'invalid attached file ID : ' . $fileId );
			}
			
			$imgurl = App::getRootDirName() . '/viewimg.php?entryname=' . $entryName . '&entryid=' . $entryId . '&fileid=' . $fileId. '&/' . rawurlencode( $orgFileName );
			return $imgurl;
		}
		
		public static function createFileUrl( $entryName, $entryId, $fileId, $orgFileName )
		{
			if ( !InputValidater::IsEntryName( $entryName ) )
			{
				cmsd_abort_proc( 'invalid entry name : ' . $entryName );
			}
			
			if ( !InputValidater::IsEntryId( $entryId ) )
			{
				cmsd_abort_proc( 'invalid entry ID : ' . $entryId );
			}
			
			if ( !InputValidater::IsAttachedFileId( $fileId ) )
			{
				cmsd_abort_proc( 'invalid attached file ID : ' . $fileId );
			}
						
			$imgurl = App::getRootDirName() . '/dlfile.php?entryname=' . $entryName . '&entryid=' . $entryId . '&fileid=' . $fileId. '&/' . rawurlencode( $orgFileName );
			return $imgurl;
		}
			
	}
	
	
	/* --------------------------------------------------------------------
	 * Xml操作ユーティリティ。
	 * 
	 */		
	 class XmlUtil
	 {
		
		/* ----------------------------------------------------------------
		 * ノード直下のテキストノードを削除する。
		 */
		public static function deleteTextNode( &$xmlNode )
		{
			foreach( $xmlNode->child_nodes() as $xmlNodeChild )
			{
				if ( $xmlNodeChild->node_type() == XML_TEXT_NODE )
				{
					$xmlNodeChild->unlink_node();
				}
			}
		}
	}
	
	/* --------------------------------------------------------------------
	 * 入力チェック（セキュリティ対策）用ユーティリティ。
	 * 
	 */		
	class InputValidater
	{
		public static function IsEntryId( $entryID )
		{
			return preg_match( '/^[^.\/][^\/]*$/', $entryID );
			//return ereg( '^[^./][^/]*$', $entryID );
		}
		
		public static function IsEntryName( $entryName )
		{
			return preg_match( '/^[^.\/][^\/]*$/', $entryName );
			//return ereg( '^[^./][^/]*$', $entryName );
		}
		
		public static function IsSchemaName( $schemaName )
		{
			return preg_match( '/^[^.\/][^\/]*$/', $schemaName );
			//return ereg( '^[^./][^/]*$', $schemaName );
		}

		public static function IsDesignName( $designName )
		{
			return preg_match( '/^[^.\/][^\/]*$/', $designName );
			//return ereg( '^[^./][^/]*$', $designName );
		}
		
		public static function IsAttachedFileId( $fileid )
		{
			return preg_match( '/^[^.\/][^\/]*$/', $fileid );
			//return ereg( '^[^./][^/]*$', $fileid );
		}
	}
	
	/* -----------------------------------------------------------
	 * [共通関数]
	 * ファイル名から拡張子を取り出す。
	 */
	function getfileext( $filename )
	{
		// '.'で区切って一番最後のトークンを取り出せば、それが拡張子のはず・・。
		$tokens = explode( '.', $filename );
		return array_pop( $tokens );
	}
	
	function gettmpfileext( $filename )
	{
		// '.'で区切って一番最後のトークンを取り出せば、それが拡張子のはず・・。
		// tmpファイルの場合は、xxx.xxx.jpg.tmp のようになっているので、最後から２番目を返す。
		$tokens = explode( '.', $filename );
		array_pop( $tokens );
		return array_pop( $tokens );
	}
	
	
	/* ------------------------------------------------------------------
	 * SiteConfigクラス
	 */
	$gEntryConfigArr = array();	// CMSEntryConfigクラスのキャッシュ用グローバル変数。SiteConfigが管理する。
	$gOutputConfig = null; 	// CMSOutputConfigクラスのキャッシュ用グローバル変数。SiteConfigが管理する。
	class CMSSiteConfig
	{
	 	function CMSSiteConfig()
		{
			global $gEntryConfigArr;
			if ( empty( $gEntryConfigArr ) )
			{
				$this->loadFromFile();
			}
		}
		
		function loadFromFile()
		{
			global $gEntryConfigArr;
			global $gOutputConfig;
			
			// キャッシュを初期化
			$gEntryConfigArr = array();
			
			// Site.Configより指定されたエントリ設定情報を取得する。
			$xmlDocSite = domxml_open_file( App::getSiteConfigFilePath() );
			
			// XMLの読み込みに成功したか？
			if ( empty( $xmlDocSite ) )
			{
				cmsd_abort_proc( App::getSiteConfigFilePath() . " の読み込みに失敗しました。ファイルが正しいXML形式ではない可能性があります。" );
			}
			
			$xpath = $xmlDocSite->xpath_new_context();
			
			// entry情報
			$xmlEntryConfigArr = $xpath->xpath_eval( "//entry" );
			foreach( cmsd_get_nodeset( $xmlEntryConfigArr ) as $xmlEntryConfig )
			{
				$entryConfig = new CMSEntryConfig( $xmlEntryConfig );
				$gEntryConfigArr[ $entryConfig->entryName ] = $entryConfig;
			}
			
			// encoding情報
			$xmlOutputConfigArr = $xpath->xpath_eval("//output" );
			if ( count( cmsd_get_nodeset( $xmlOutputConfigArr ) ) > 0 )
			{
				$gOutputConfig = new CMSOutputConfig( $xmlOutputConfigArr->nodeset[0] );
			}
			else
			{
				$gOutputConfig = new CMSOutputConfig();
			}
			
		}
		
		function &getEntryConfig( $entryName )
		{
			global $gEntryConfigArr;
			return $gEntryConfigArr[ $entryName ];
		}
		
		function &getOutputConfig()
		{
			global $gOutputConfig;
			return $gOutputConfig;
		
		}
	 
	 }
	 
	 
	 // CMSD文字エンコーディング設定クラス
	 class CMSOutputConfig
	 {
	 	var $encoding = "";			// ブラウザへ返す文字コード
		var $disableXsltEncoding = false;	// XSLTライブラリによる出力文字コード変換処理を無効にする。
		var $metaTagReplacement = true;		// 最終出力文字列中のメタタグのcharsetを自動的に出力文字コードに変換する機能をONにする。
		var $xsltLibNo = 0;					// XSLTライブラリ番号。
		
	 	function CMSOutputConfig( $xmlOutputConfig = null )
		{
			$this->encoding = gCMSDInternalEncoding(); // 省略値
			if ( ! is_null( $xmlOutputConfig ) )
			{
				$this->encoding = parseString( $xmlOutputConfig->get_attribute( 'encoding' ), $this->encoding );
				//$this->script = parseString( $xmlOutputConfig->get_attribute( 'script' ), $this->script );
				$this->disableXsltEncoding = parseBoolean( $xmlOutputConfig->get_attribute( 'disable-xslt-encoding' ), false );
				$this->metaTagReplacement = parseBoolean( $xmlOutputConfig->get_attribute( 'meta-tag-replacement' ), true );
				$this->xsltLibNo = parseNumeric( $xmlOutputConfig->get_attribute( 'xslt-libno' ), 0 );
			}
		}

		
		function getEncoding()
		{
			return $this->encoding;
		}
		
		function setEncoding( $newValue )
		{
			$this->encoding = $newValue;
		}
		
		function getDisableXsltEncoding()
		{
			return $this->disableXsltEncoding;
		}
		
		function setDisableXsltEncoding( $newValue )
		{
			$this->disableXsltEncoding = $newValue;
		}
		
		function getMetaTagReplacement()
		{
			return $this->metaTagReplacement;
		}
		
		function setMetaTagReplacement( $newValue )
		{
			$this->metaTagReplacement = $newValue;
		}
		
		function getXsltLibNo()
		{
			return $this->xsltLibNo;
		}
		
		function setXsltLibNo( $newValue )
		{
			$this->xsltLibNo = $newValue;
		}
		
	 }
	 
	 // CMSDエントリ定義クラス
	 class CMSEntryConfig
	 {
	 	var $entryName;			// エントリフォルダ名
		var $entryType;			// エントリタイプ。省略時は"entry"。
		var $schemaName;		// スキーマ名
		var $caption;			// 見出し（キャプション）
		var $entryPageUrl;		// エントリ１件表示用URL（絶対パス）
		var $listPageUrl;		// エントリ一覧表示用URL（絶対パス）
		var $publishControl;	// 公開方法
		var $imageCacheList = array();	// 画像キャッシュ指定リスト
		var $permitList = array();		// 権限情報リスト
		var $datasourceList = array();	// データソースリスト
		
	 	function CMSEntryConfig( $xmlEntryConfig )
		{
			$this->entryName = $xmlEntryConfig->get_attribute( 'name' );
			$this->entryType = $xmlEntryConfig->get_attribute( 'type' );
			$this->schemaName = $xmlEntryConfig->get_attribute( 'schema' );
			$this->caption = file2code( $xmlEntryConfig->get_attribute( 'caption' ) );
			$this->entryPageUrl = $xmlEntryConfig->get_attribute( 'entrypageurl' );
			$this->listPageUrl = $xmlEntryConfig->get_attribute( 'listpageurl' );
			$this->publishControl = $xmlEntryConfig->get_attribute( 'publishcontrol' );
			
			// チェック
			
			// name属性
			if ( empty( $this->entryName ) )
			{
				cmsd_abort_proc( "site.config.xmlのエントリ定義にname属性が無いか、空白が指定されています。" );
			}
			
			
			if ( $this->entryType != 'separator' ) // type属性が"separator"の場合は、セパレータなので以下のチェックを行わない。
			{
			
				if ( strstr( $this->entryName, ' ' ) !== false )
				{
					cmsd_abort_proc( "site.config.xmlのエントリ定義にて、name属性に空白が使用されています。" );
				}
				
				// schema属性
				if ( empty( $this->schemaName ) )
				{
					cmsd_abort_proc( "site.config.xmlのエントリ定義にschema属性が無いか、空白が指定されています。" );
				}
	
				if ( strstr( $this->schemaName, ' ' ) !== false )
				{
					cmsd_abort_proc( "site.config.xmlのエントリ定義にて、schema属性に空白が使用されています。" );
				}
				
				// caption属性
				if ( empty( $this->caption ) )
				{
					cmsd_abort_proc( "site.config.xmlのエントリ定義にcaption属性が無いか、空白が指定されています。" );
				}
				
				// publish
				if ( empty( $this->publishControl ) )
				{
					$this->publishControl = 'on-off';	// 省略時は"on-off"。
				}
				else if ( $this->publishControl != 'none' and $this->publishControl != 'on-off' and $this->publishControl != 'full' )
				{
					cmsd_abort_proc( "site.config.xmlのエントリ定義にて、publish属性に不正な値'{$this->publish}'が指定されています。" );
				}
			}
			
			// imagecache要素の読み込み
			$xmlImageCacheList = $xmlEntryConfig->get_elements_by_tagname( 'imagecache' );
			foreach ( $xmlImageCacheList as $xmlImageCache )
			{
				$this->imageCacheList[] = new CMSEntryConfigImageCache( $xmlImageCache );
			}
			
			//print "EntryConfig '{$this->entryName}' is loaded with publish = '{$this->publish}'<br />";
			
			// permit要素の読み込み
			$xmlPermitList = $xmlEntryConfig->get_elements_by_tagname( 'permit' );
			foreach ( $xmlPermitList as $xmlPermit )
			{
				$this->permitList[] = new CMSEntryConfigPermit( $xmlPermit );
			}
			
			// datasource要素の読み込み
			$xmlDatasourceList = $xmlEntryConfig->get_elements_by_tagname( 'datasource' );
			foreach ( $xmlDatasourceList as $xmlDatasource )
			{
				$this->datasourceList[] = new CMSEntryConfigDatasource( $xmlDatasource );
			}
			
		}
		
		// データソース定義の取得（提供先の項目名で検索）
		function getDatasource( $itemName ) 
		{
			$targetDatasource = null;
			foreach( $this->datasourceList as $datasource )
			{
				if ( $datasource->for === $itemName )
				{
					$targetDatasource = $datasource;
					break;
				}
			}
			return $targetDatasource;
		
		}
		
		function getDatasourceList()
		{
			return $this->datasourceList;
		}
		
		function canShowInContentList( $user )
		{
			$able = false; 	// 初期値
			
			if ( count( $this->permitList ) <= 0 )
			{
				// permit要素が一つも無かった場合、全ての人に対して全ての権限が与えられる。（過去の仕様との整合性の為）
				$able = true;
			}
			else
			{
				// mineに対するlist権限
				$ableMine = $this->canAction( $user, 'list', 'mine' );
				// othersに対するlist権限
				$ableOthers = $this->canAction( $user, 'list', 'others' );
				
				// どちらかがtrueなら、コンテンツ一覧へ表示する
				$able = ( $ableMine || $ableOthers );
			
			}
			
			return $able;
		}
		
		
		function canShowInContentList2( $user )
		{
			$permission = 'none'; 	// 初期値
			
			if ( count( $this->permitList ) <= 0 )
			{
				// permit要素が一つも無かった場合、全ての人に対して全ての権限が与えられる。（過去の仕様との整合性の為）
				$permission = 'permit';
			}
			else
			{
				// 全ての権限情報から１つ１つパーミッションを取得し、none < permit < deny の順で上書きする。（例：permitはnoneでは上書きできない）
				foreach ( $this->permitList as $permit )
				{
					// 各権限情報からパーミッションを取得する。
					$permission_temp = $permit->getPermissionShowInContentList( $user );
					
					//print "permission = " . $permission_temp . "   ";
					
					switch ( $permission_temp )
					{
						case 'none' :
							// 何もしない。
							break;
						
						case 'permit' :
							// 現在がnoneの場合のみ、上書きする。
							if ( $permission === 'none' )
							{
								$permission = $permission_temp;
							}
							break;
						
						case 'deny' :
							// 強制的に上書きする。
							$permission = $permission_temp;
							break;
					}
					
					// denyだった場合、これ以上回す理由がないのでループを終了する。
					if ( $permission === 'deny' )
					{
						break;
					}
				}
			
			}
			
			return ( $permission === 'permit' );
			
		}
		
		// 指定したactionに対する権限を持っているか？
		// $actionName : list, add, delete, edit, publish
		// $target : 'mine' or 'others'
		// Result: 可能:true  不可能:false
		function canAction( $user, $actionName, $target )
		{
			$permission = 'none'; 	// 初期値
			
			if ( count( $this->permitList ) <= 0 )
			{
				// permit要素が一つも無かった場合、全ての人に対して全ての権限が与えられる。（過去の仕様との整合性の為）
				$permission = 'permit';
			}
			else
			{
				// 全ての権限情報から１つ１つパーミッションを取得し、none < permit < deny の順で上書きする。（例：permitはnoneでは上書きできない）
				foreach ( $this->permitList as $permit )
				{
					// 各権限情報からパーミッションを取得する。
					$permission_temp = $permit->getPermission( $user, $actionName, $target );
					
					switch ( $permission_temp )
					{
						case 'none' :
							// 何もしない。
							break;
						
						case 'permit' :
							// 現在がnoneの場合のみ、上書きする。
							if ( $permission === 'none' )
							{
								$permission = $permission_temp;
							}
							break;
						
						case 'deny' :
							// 強制的に上書きする。
							$permission = $permission_temp;
							break;
					}
					
					// denyだった場合、これ以上回す理由がないのでループを終了する。
					if ( $permission === 'deny' )
					{
						break;
					}
				}
			
			}
			
			return ( $permission === 'permit' );
			
		}
		
	 }
	 
	 class CMSEntryConfigImageCache
	 {
		var $width = 0;
		var $height = 0;
		
		function CMSEntryConfigImageCache( $xmlEntryConfigImageCache )
		{
			$this->width = parseString( $xmlEntryConfigImageCache->get_attribute('width') );
			$this->height = parseString( $xmlEntryConfigImageCache->get_attribute('height') );
			
			if ( empty( $this->width ) )
			{
				$this->width = 0;
			}
			
			if ( empty( $this->height ) )
			{
				$this->height = 0;
			}
			
		}
		
		function isTarget( $width, $height )
		{
			$isTarget = false;
			
			$matchWidth = false;
			$matchHeight = false;
			
			if ( $this->width === 'all' )	// ※ 数値と文字列を比較した場合、文字列が数値として評価される為、単純に"=="で比較すると0の場合に一致してしまう。
			{
				$matchWidth = true;	// 全ての場合
			}
			else if ( empty( $this->width ) ) // 0 又は省略されていた時
			{
				$matchWidth = empty( $width );	// 0 又は省略されていた時
			}
			else
			{
				$matchWidth = ( $this->width === $width );	// 数値が一致した時
			}
			
			
			if ( $this->height === 'all' )
			{
				$matchHeight = true;	// 全ての場合
			}
			else if ( empty( $this->height ) ) // 0 又は省略されていた時
			{
				$matchHeight = empty( $height );	// 0 又は省略されていた時
			}
			else
			{
				$matchHeight = ( $this->height === $height );	// 数値が一致した時
			}
			
			$isTarget = ( $matchWidth and $matchHeight );
			
			/*
			if ( empty( $this->width ) and empty( $this->height ) )
			{
				// 両方指定なしのキャッシュ指定は無効
				
			}
			elseif ( empty( $this->width ) )
			{
				$isTarget = ( empty( $width ) and $this->height == $height );
			}
			elseif ( empty( $this->height ) )
			{
				$isTarget = ( empty( $height ) and $this->width == $width );
			}
			elseif ( $this->width == 'all' and $this->height == 'all' )
			{
				$isTarget = true;
			}
			elseif ( $this->width == 'all' )
			{
				$isTarget = ( $this->height == $height );
			}
			elseif ( $this->height == 'all' )
			{
				$isTarget = ( $this->width == $width );
			}
			else
			{
				$isTarget = ( $this->width == $width and $this->height == $height );
			}
			*/
			
			return $isTarget;
			
		}
	 }
	 
	 /* ---------------------------------------------------------------
	  * Permitクラス　[2008.04 ユーザー権限]
	  * 
	  * 
	  */
	 class CMSEntryConfigPermit
	 {
	 	var $users = array();
	 
	 	var $mine = array();		// 自分が作成したエントリに対するパーミッション（例：'list,read,add,edit'）
		var $others = array();	// 他人が作成したエントリに対するパーミッション
		
		var $maxrow = 0;	// 未実装
		
		function CMSEntryConfigPermit( $xmlPermit )
		{
			// mineとothersの省略値は'all'。
			$mine = parseString( $xmlPermit->get_attribute( 'mine' ), 'all' );
			$others = parseString( $xmlPermit->get_attribute( 'others' ), 'all' );
			
			$this->mine = explode( ',', $mine );
			$this->others = explode( ',', $others );
			
			// 'deny'が指定されていたら、それ以外は無効（denyが最優先される）
			if ( in_array( 'deny', $this->mine ) ) $this->mine = array( 'deny' );
			if ( in_array( 'deny', $this->others) ) $this->others = array( 'deny' );
			
			// 'none'が指定されていたら、それ以外は無効（allよりnoneの方を優先する）
			if ( in_array( 'none', $this->mine ) ) $this->mine = array( 'none' );
			if ( in_array( 'none', $this->others) ) $this->others = array( 'none' );
			
			// 'all'が指定されていたら、それ以外は無効
			if ( in_array( 'all', $this->mine ) ) $this->mine = array( 'all' );
			if ( in_array( 'all', $this->others) ) $this->others = array( 'all' );
			
			// 対象ユーザーの読み込み
			$users = parseString( $xmlPermit->get_attribute( 'users' ) );
			
			if ( strlen( $users ) <= 0 )
			{
				cmsd_abort_proc( 'permit要素にはusers属性に1つ以上のユーザー名の指定が必要です。' );
			}
			
			// カンマ区切りのユーザーIDリストを分解して配列にする。
			$this->users = explode( ',', $users );
			
		}
		
		function isTargetUser( $userId )
		{
			// usersの配列中にuserIdが存在したらtrueを返す。(publisher)を含む場合には、無条件でtrueを返す。
			return in_array( '(publisher)', $this->users ) || in_array( $userId, $this->users );
		}
		
		function GetPermissionShowInContentList( $user )
		{
			$pemission = 'none';	// 初期値（ダミー）
			
			// mineに対するlist権限
			$permissionMine = $this->GetPermission( $user, 'list', 'mine' );
			// othersに対するlist権限
			$permissionOthers = $this->GetPermission( $user, 'list', 'others' );
			
			// どちらもdenyなら、コンテンツ一覧への表示もdeny
			if ( $permissionMine === 'deny' && $permissionOthers === 'deny' )
			{
				$permission = 'deny';
			}
			// どちらかがpermitなら、コンテンツ一覧への表示もpermit
			elseif ( $permissionMine === 'permit' || $permissionOthers === 'permit' )
			{
				$permission = 'permit';
			}
			// それ以外はnone。
			else
			{
				$permission = 'none';
			}
			
			return $permission;
		
		}

		function innerGetPermission( $actionName, $permittedActions )
		{
		
			$permission = 'none'; // 初期値はダミー
			
			if ( in_array( 'all', $permittedActions ) )
			{
				$permission = 'permit';
			}
			elseif ( in_array( 'none', $permittedActions ) )
			{
				$permission = 'none';
			}
			elseif ( in_array( 'deny', $permittedActions ) )
			{
				$permission = 'deny';
			}
			else
			{
				// allでもnoneでもない場合。
				switch ( $actionName )
				{
					case 'list' :
						// -listが含まれてる場合以外は全て permit。
						if ( in_array( '-list', $permittedActions ) )
						{
							$permission = 'deny';
						}
						else
						{
							$permission = 'permit';
						}
						
						break;
					
					case 'read' :	// 今は実装しない。
					
						break;
					
					case 'publish' : // 今は実装しない。
						break;
					
					case 'edit' :
					case 'add' :
					case 'delete' :
						// そのアクションが含まれていれば permit、-アクションが含まれていれば、deny、それ以外なら none。
						if ( in_array( $actionName, $permittedActions ) )
						{
							$permission = 'permit';
						}
						elseif ( in_array( '-'.$actionName, $permittedActions ) )
						{
							$permission = 'deny';
						}
						else
						{
							$permission = 'none';
						}
						
						break;
				
				}
			}
			
			return $permission;
		}
		
		// パーミッションの取得
		// $target : 'mine' or 'others'
		// 'permit","none","deny" のいずれかを返す
		function getPermission( $user, $actionName, $target )
		{
			$permission = "none"; // 初期値（ダミー）
			
			if ( $user->isAdministrator() )
			{
				// Administratorならば、無条件にpermitを返す。
				$permission = 'permit';
			}
			elseif ( ! $this->isTargetUser( $user->getUserId() ) )
			{
				// このPermitの対象ユーザーでなければnoneを返す。
				$permission = 'none';
			}
			else
			{
				// ユーザーがAdministrator以外で、Permitの対象ユーザーの場合の処理
				switch ( $target )
				{
					case 'mine' : 
						// mineに対するパーミッションを取得
						$permission = $this->innerGetPermission( $actionName, $this->mine );
						break;
					
					case 'others' :
						// othersに対するパーミッションを取得
						$permission = $this->innerGetPermission( $actionName, $this->others );
						break;
					
				}
			}
			
			return $permission;
		
		}
		
	}
	
	
	/* --------------------------------------------------------------------
	 * エントリ定義のデータソースクラス。
	 * 現時点では、menu項目の外部エントリフォルダ参照用。将来的に、Web-APIからのデータ取得等にも使えると面白そうだが…。
	 * 2014.8.25 新規追加。
	 */		
	class CMSEntryConfigDatasource
	{
		var $for = "";		// データソース提供先
		var $type = "";		// データソース種別　entrylist:外部エントリフォルダ
		var $from = "";		// データソース提供元（type=entrylist用）
		var $label = "";	// データソース提供元がentrylistだった場合にmenuitemのラベルとして用いる項目名（type=entrylist用）
		var $menuItems = array();	// データソース提供元がentrylistだった場合にmenuitemの選択肢を保持する配列。index:id, content:label
		var $embedding = TRUE;	// 子要素の埋め込みを行うかどうか（type=entrylist用）
		
		function CMSEntryConfigDatasource( $xmlEntryConfigDatasource )
		{
			$this->for = parseString( $xmlEntryConfigDatasource->get_attribute('for') );
			$this->type = parseString( $xmlEntryConfigDatasource->get_attribute('type') );
			
			if ( $this->type == 'inline' ) {
				
				// menuitem要素の読み込み
				$xmlMenuItemList = $xmlEntryConfigDatasource->get_elements_by_tagname( 'menuitem' );
				foreach ( $xmlMenuItemList as $xmlMenuItem )
				{
					$this->menuItems[ file2code( $xmlMenuItem->get_attribute( 'id' ) ) ] = file2code( $xmlMenuItem->get_content() );
				}
			}
			
			if ( $this->type == 'entrylist' ) {
				$this->from = parseString( $xmlEntryConfigDatasource->get_attribute('from') );
				$this->label = parseString( $xmlEntryConfigDatasource->get_attribute('label') );
				$this->embedding = ( strcasecmp( parseString ( $xmlEntryConfigDatasource->get_attribute('embedding') ), 'false' ) === 0 ) ? FALSE : TRUE;
				
				// 対象となるエントリフォルダのindexファイルを読み込む。
				// indexファイルが無い場合、エラーを出力する。
				if ( ! file_exists( App::createEntryFilePath( $this->from, 'index' ) ) )
				{
					cmsd_abort_proc( "site.config.xml: データソース'{$this->for}'のfrom属性に指定されたエントリフォルダ'{$this->from}'のindexファイルが存在しません。site.config.xmlをご確認頂くか、対象のエントリフォルダのデータをご確認ください。" );
					
				}

				// エントリindexのXMLを読み込む。
				$xmlDocEntryIndex = domxml_open_file( App::createEntryFilePath( $this->from, 'index' ) );
				$xpath = $xmlDocEntryIndex->xpath_new_context();
				
				// 全エントリデータの取得
				$xmlEntryList = $xpath->xpath_eval( "/entrylist/entry" );
				$xmlEntryArr = cmsd_get_nodeset( $xmlEntryList );
				
				// indexファイルから各ファイル名を取得し、該当ファイルのDOMを開いて要約を表示する。
				foreach( $xmlEntryArr as $entryIndex=>$xmlEntryRef )
				{
					$menuitem_id = parseString( $xmlEntryRef->get_attribute( 'id' ) );
					
					$xmlLabelArr = cmsd_get_nodeset( $xpath->xpath_eval( "/entrylist/entry[@id='{$menuitem_id}']/groupkey[@key='{$this->label}']" ));
					if ( empty( $xmlLabelArr ) ) { //or $xmlLabelNodeList->length <= 0 ) {
						//cmsd_abort_proc( "site.config.xml: データソース'{$this->for}'のfrom属性に指定されたエントリフォルダ'{$this->from}'のindexファイル中に、label属性で指定された'{$this->label}'のデータが存在しません。site.config.xmlをご確認頂くか、対象のエントリフォルダのデータをご確認ください。labelに指定する項目は、スキーマのtitle属性に指定された状態でデータが保存されている必要があります。" );
						$menuitem_label = "";
					}
					else
					{
						$menuitem_label = file2code( $xmlLabelArr[0]->get_content() );
					}
					
					/*
					$xmlLabelNodeList = $xmlEntryRef->get_elements_by_tagname( $this->label );
					
					if ( empty( $xmlLabelNodeList ) ) { //or $xmlLabelNodeList->length <= 0 ) {
					cmsd_abort_proc( "site.config.xml: データソース'{$this->for}'のfrom属性に指定されたエントリフォルダ'{$this->from}'のindexファイル中に、label属性で指定された'{$this->label}'のデータが存在しません。site.config.xmlをご確認頂くか、対象のエントリフォルダのデータをご確認ください。labelに指定する項目は、スキーマのtitle属性に指定された状態でデータが保存されている必要があります。" );
					}
					$menuitem_label = file2code( $xmlLabelNodeList[0]->get_content() );
					*/
					
					// 各エントリについて、エントリIDとラベル項目のデータを対として配列要素に保存する。
					$this->menuItems[ $menuitem_id ] = $menuitem_label;
				}
				
			}
			
			
		}
		
	}
	
	
	/* --------------------------------------------------------------------
	 * リロード対策クラス。
	 * リロード用トークンの生成／管理を行う。全てクラスメソッド。
	 */	
	class ReloadToken
	{
		
		public static function getCurrentToken()
		{
			return $_SESSION[ ReloadToken::getTokenKey() ];		
		}
		
		public static function getRequestToken()
		{
			$token = $_REQUEST[ ReloadToken::getTokenKey() ];
			
			return $token;
						
		}
		
		public static function isReload()
		{
		
			if ( ReloadToken::getCurrentToken() === ReloadToken::getRequestToken() )
			{
				return false;
			}
			else
			{
				return true;
			}
		}
		
		public static function newToken()
		{
			$newToken = md5(uniqid(rand(),1));
			
			$_SESSION[ ReloadToken::getTokenKey() ] = $newToken;
			
			return $newToken;
		}

		public static function embed()
		{
			$token = ReloadToken::getCurrentToken();
			
			return '<input type="hidden" name="' . ReloadToken::getTokenKey() . '" value="' . $token . '" />' . "\n";
		}
		
		public static function embedToXmlDoc( &$xmlDoc )
		{
			$token = ReloadToken::getCurrentToken();
			
			$xmlToken = $xmlDoc->create_element('reloadtoken');
			$xmlToken->set_attribute( 'key', ReloadToken::getTokenKey() );
			$xmlToken->set_attribute( 'value', $token );
			
			$xmlRoot = $xmlDoc->document_element();
			$xmlRoot->append_child( $xmlToken );
		}
		
		public static function getTokenKey()
		{
			// "."が含まれるとダメらしい・・。
			return 'jp_al-design_ReloadToken';
		}	
		
	}
	
?>