<?php
	/*=======================================================================
	 * copyright (c) AL-Design Works 
	 * http://cms.al-design.jp/
	 * このソースファイルは CMS Designer の一部です。
	 * 詳しいライセンス規約についてはウェブサイトを参照してください。
	 *=======================================================================
	 */

	//require_once( dirname(__FILE__) . "/app.php.inc" );
	require_once( dirname(__FILE__) . "/manager.util.php.inc" );
	
	require_once( dirname(__FILE__) . "/manager.base.php.inc" );

	/* --------------------------------------------------------------------
	 * 整数入力項目クラス。
	 * 
	 */ 
	class IntEntryItem extends BaseEntryItem
	{
		var $max = 0;
		var $min = 0;
		var $size = 4;
		
		/* ----------------------------------------------------------------
		 * コンストラクタ。
		 * $xmlSchemaData : 対応するスキーマDataのXML要素。
		 */
		function IntEntryItem( $xmlSchemaData, $entryName, $entryId  )
		{
			// ベースコンストラクタの呼び出し。
			parent::BaseEntryItem( $xmlSchemaData, $entryName, $entryId  );
			
			$this->max = (int)( $xmlSchemaData->get_attribute('max') ); 
			$this->min = (int)( $xmlSchemaData->get_attribute('min') );
			
			$size = (int)( $xmlSchemaData->get_attribute('size') );
			if ( $size <= 0 )
			{
				$size = 4;	// 初期値
			}
			$this->size = $size;
			
		}
		
		/* ----------------------------------------------------------------
		 * 当項目のXMLノードを新規作成する。
		 * $xmlDocParentEntry : エントリのDOMオブジェクト。
		 * $setInitData : 初期値。trueが指定されていた場合、スキーマで指定された値が設定される。デフォルト値はtrue。
		 * return : 新規XMLノード。
		 */
		function createNew( $xmlDocParentEntry, $setInitData = true )
		{
			$newNode = parent::createNew( $xmlDocParentEntry, $setInitData );
			
			return $newNode;
		}
		
		
		/* ----------------------------------------------------------------
		 * 入力内容でエントリ項目を更新する。
		 * $xpathEntry : エントリのXPathオブジェクト。
		 * $currentEntry : 現在のエントリ位置を表すXPath文字列。
		 */
		function update( $xpathEntry, $currentEntry )
		{
			// エントリXMLを取得。
			$xmlEntryDataList = $xpathEntry->xpath_eval( $currentEntry . '/' . $this->dataName );
			$xmlEntryData = $xmlEntryDataList->nodeset[0];
			
			/*
			if ( count($xmlEntryDataList) > 0 )
			{
				$xmlEntryData = $xmlEntryDataList->nodeset[0];
			}
			else
			{
				// 該当のデータ項目が存在しない場合、スキーマと同期を取る為、新規ノードを生成する。
				$xmlParentDataArr = $xpathEntry->xpath_eval( $currentEntry );
				$xmlParentData = $xmlParentDataArr->nodeset[0];
				$xmlDocEntry = $xmlParentData->owner_document();
				$xmlEntryData = $this->createNew( $xmlDocEntry );
				$xmlParentData->append_child( $xmlEntryData );
			}
			*/

			// "name"属性に対応するPOSTが存在すれば、エントリに反映する。
			if ( isset( $_POST[ $this->dataName ] ) )
			{
				$datavalue = $_POST[ $this->dataName ];
				
				// 数値チェック
				if ( ! is_numeric( $datavalue ) )
				{
					$this->errorMessage = sprintf( "%s に数値以外が入力されました。", $this->dataCaption );
					return false;
				}
				
				// 範囲チェック
				if ( 
					( $datavalue < $this->min ) or 
					( ( $datavalue > $this->max ) and ! empty( $this->max ) ) // maxが0ならば、上限チェックはしない。
				)
				{
					$this->errorMessage = sprintf( "%s が %d ～ %d の範囲を超えました", $this->dataCaption, $this->min, $this->max );
					return false;
				}
				
				// まず、テキストノードを削除する（削除しないと、set_contentではテキストノードが増えるだけ）。
				$xmlEntryData = $xmlEntryDataList->nodeset[0];
				EntryUtil::deleteTextNode( $xmlEntryData );
				
				// テキストノードに新しい値をsetする。
				$xmlEntryData->set_content( fileconv( $datavalue ) );
				//print( $dataname . " = " . htmlconv( $datavalue ) . "<br>\n" );
			}
			
			return true;
		
		}
		
		/* ----------------------------------------------------------------
		 * フォーム項目を描画するHTMLを返す。
		 * $xpathEntry : エントリのXPathオブジェクト。
		 * $currentEntry : 現在のエントリ位置を表すXPath文字列。
		 */
		function renderFormItem( $xpathEntry, $currentEntry )
		{
			$xmlEntryDataArray = $xpathEntry->xpath_eval( $currentEntry . '/' . $this->dataName );
			$value = $xmlEntryDataArray->nodeset[0]->get_content();
			
			$strFormHTML = '';
			
			$strFormHTML .= '<input type="text" size="4" name="' . $this->dataName . '" value="' . $value . '" /><br>' . "\n";
			
			return $strFormHTML;
		}
		

		/* ----------------------------------------------------------------
		 * フォーム項目を描画するHTMLを返す。
		 * $xpathEntry : エントリのXPathオブジェクト。
		 * $currentEntry : 現在のエントリ位置を表すXPath文字列。
		 */
		function renderFormItemXML( $xpathEntry, $currentEntry, &$xmlOutputEntryData )
		{
			$xmlEntryDataArray = $xpathEntry->xpath_eval( $currentEntry . '/' . $this->dataName );
			$value = $xmlEntryDataArray->nodeset[0]->get_content();

			// リクエストに値があれば、その値を再表示する（エラー時に前回入力値が保証されるようにする）
			if ( isset( $_POST[ $this->dataName ] ) )
			{
				$value = $_POST[ $this->dataName ];
			}

			$xmlOutputEntryData->set_attribute('size', $this->size );
			$xmlOutputEntryData->set_content( $value );
		}				
		
	}

?>