<?php
	/*=======================================================================
	 * copyright (c) AL-Design Works 
	 * http://cms.al-design.jp/
	 * このソースファイルは CMS Designer の一部です。
	 * 詳しいライセンス規約についてはウェブサイトを参照してください。
	 *=======================================================================
	 */

	require_once( dirname(__FILE__) . "/version.php.inc" );
	require_once( dirname(__FILE__) . "/app.php.inc" );
	require_once( dirname(__FILE__) . "/manager.util.php.inc" );
	require_once( dirname(__FILE__) . "/manager.factory.php.inc" );
	require_once( dirname(__FILE__) . "/xmlutil.php.inc" );
	require_once( dirname(__FILE__) . "/manager.header.php.inc" );
	require_once( dirname(__FILE__) . "/cache.php.inc" );


	// エントリクラス
	class CMSEntry
	{
	
		public static function IsEntryPublished( &$xmlEntryRoot )
		{
			// 年月日時分
			$sYear = sprintf( "%04d", date('Y') );
			$sMonth = sprintf( "%02d", date('m') );
			$sDay = sprintf( "%02d", date('d') );
			$sHour = sprintf( "%02d", date('H') );
			$sMinute = sprintf( "%02d", date('i') );

			//$strToday = $sYear . '-' . $sMonth . '-' . $sDay . ' ' . $sHour . ':' . $sMinute;
			$strToday = $sYear . $sMonth . $sDay . $sHour . $sMinute;
			
			// 「公開」かつ、開始日時≦現在日時≦終了日時 のものを抽出する条件式。
			// 公開フラグや開始日時、終了日時が存在しなかった頃のバージョンとの互換性の為に、それらが無いエントリも考慮した条件式にする。
			// ※XPathにて文字列の大小比較が出来ない事が判明。但し、数値文字列の場合のみ比較可能だったので、:や-などを外して比較することにした。
			
			$startDate = $xmlEntryRoot->get_attribute('startdate');
			$endDate = $xmlEntryRoot->get_attribute('enddate');
			$visible = $xmlEntryRoot->get_attribute('visible');
			
			// まず、visibleが空ならば、古いエントリの為開始や終了も空ということになるので無条件に公開。
			// 次に、もしvisibleがFalseなら、無条件に非公開。開始日時＞現在日時（公開開始前）でも無条件に非公開。終了日時≦現在日時（公開終了後）でも、無条件に非公開。
			if ( empty( $visible ) or  ! ( $visible === 'False' or $startDate > $strToday or $endDate <= $strToday ) )	// 対応するXPath式と同じ形式にした。
			{
				return true;
			}
			else
			{
				return false;
			}
			
		}
		
		public static function GetEntryPublishState( &$xmlEntryRoot )
		{
			// 年月日時分
			$sYear = sprintf( "%04d", date('Y') );
			$sMonth = sprintf( "%02d", date('m') );
			$sDay = sprintf( "%02d", date('d') );
			$sHour = sprintf( "%02d", date('H') );
			$sMinute = sprintf( "%02d", date('i') );

			//$strToday = $sYear . '-' . $sMonth . '-' . $sDay . ' ' . $sHour . ':' . $sMinute;
			$strToday = $sYear . $sMonth . $sDay . $sHour . $sMinute;
			
			// 「公開」かつ、開始日時≦現在日時≦終了日時 のものを抽出する条件式。
			// 公開フラグや開始日時、終了日時が存在しなかった頃のバージョンとの互換性の為に、それらが無いエントリも考慮した条件式にする。
			// ※XPathにて文字列の大小比較が出来ない事が判明。但し、数値文字列の場合のみ比較可能だったので、:や-などを外して比較することにした。
			
			$startDate = $xmlEntryRoot->get_attribute('startdate');
			$endDate = $xmlEntryRoot->get_attribute('enddate');
			$visible = $xmlEntryRoot->get_attribute('visible');
			
			if ( empty( $visible ) )
			{
				$publishState = 'public';		// 公開
			}
			else if ( $visible === 'False' )
			{
				$publishState = 'private';		// 非公開
			}
			else if ( $startDate > $strToday )
			{
				$publishState = 'not yet';		// 未公開
			}
			else if ( $endDate <= $strToday )
			{
				$publishState = 'over';			// 終了
			}
			else if ( $startDate === '000000000000' and $endDate === '999999999999' )
			{
				$publishState = 'public';		// 公開
			}
			else
			{
				$publishState = 'during';		// 公開中
			}
			
			return $publishState;
		}
		
	
	}
	
	
	class CommonActionUtil
	{
		public static function XsltEntryEditor( $xmlDocOutput )
		{
			// 全画面共通情報を出力
			$xmlOutputRoot = $xmlDocOutput->document_element();
			
			// ログインユーザーID
			$authedUser = LoginManager::GetAuthedUser();
			$authedUserId = $authedUser->getUserId();
			$xmlOutputRoot->set_attribute( 'autheduserid', $authedUserId );
			
			// XSL変換
			$strFormHTML = ADXslt::process( $xmlDocOutput, dirname(__FILE__) . "/entryeditor.xsl" );
			return $strFormHTML;
		}
	}

	
	class VersionInfoAction
	{
		function procRequest()
		{
			print ( $this->renderHTMLXML() );
		}
		
		function renderHTMLXML()
		{
			// HTML生成開始	
			// DOMをを生成。
			// ルートノード'entryeditor'を生成。
			$xmlDocOutput = domxml_new_doc( '1.0' );
			$xmlOutputRoot = $xmlDocOutput->add_root( 'versioninfo' );
			$xmlOutputRoot->set_attribute( 'appname', CMSD_APP_NAME );
			$xmlOutputRoot->set_attribute( 'appversion', CMSD_APP_VERSION );
			
			// XSL変換処理
			//$strFormHTML = ADXslt::process( $xmlDocOutput, dirname(__FILE__) . "/entryeditor.xsl" );
			$strFormHTML = CommonActionUtil::XsltEntryEditor( $xmlDocOutput );
			
			return $strFormHTML;
			
		}
		
	}
	
	
	class EntryNamesForEditor
	{
		function procRequest()
		{
			// site.configが読み込み可能かチェック
			if ( ! is_readable( App::getSiteConfigFilePath() ) )
			{
				cmsd_abort_proc( App::getSiteConfigFilePath() . " が存在しないか読み込み不可になっています。" );
			}
			
			// Site.Configより指定されたエントリ設定情報を取得する。
			$xmlDocSite = domxml_open_file( App::getSiteConfigFilePath() );
			
			// XMLの読み込みに成功したか？
			if ( empty( $xmlDocSite ) )
			{
				cmsd_abort_proc( App::getSiteConfigFilePath() . " の読み込みに失敗しました。ファイルが正しいXML形式ではない可能性があります。" );
			}
			
			$xpath = $xmlDocSite->xpath_new_context();
			
			$xmlEntryConfigList = $xpath->xpath_eval( "/site/entries/entry" );
			
			print( $this->renderHTMLXML( cmsd_get_nodeset( $xmlEntryConfigList ) ) );
			
		}
		
		function renderHTMLXML( $xmlEntryConfigList )
		{
			// HTML生成開始	
			// DOMをを生成。
			// ルートノード'entryeditor'を生成。
			$xmlDocOutput = domxml_new_doc( '1.0' );
			$xmlOutputRoot = $xmlDocOutput->add_root( 'entrynames' );
			
			foreach( $xmlEntryConfigList as $xmlEntryConfig )
			{
				// 各種エントリフォルダ情報の取得
				$entryname = $xmlEntryConfig->get_attribute( 'name' );
				$entrytype = $xmlEntryConfig->get_attribute( 'type' );
				$entrycaption = file2code( $xmlEntryConfig->get_attribute( 'caption' ) );
				$entrydescription = file2code( $xmlEntryConfig->get_attribute( 'description' ) );
				
				// EntryConfigクラスを生成
				$entryConfig = new CMSEntryConfig( $xmlEntryConfig );
				$user = LoginManager::GetAuthedUser();
				
				// ログインユーザーが、該当エントリフォルダの表示権限を持っていれば、表示対象に加える。[2008.04 ユーザー権限]
				if ( $entryConfig->canShowInContentList( $user ) )
				{
					// Entry情報の出力
					$xmlOutputEntry = $xmlDocOutput->create_element('entryname');
	
					if ( strlen( $entrytype) > 0 )
					{
						$xmlOutputEntry->set_attribute('entrytype', $entrytype );
					}
					else
					{
						$xmlOutputEntry->set_attribute('entrytype', 'entryfolder' );	// 省略時は"entryfolder"とする。
					}
					
					if ( strlen( $entrycaption ) > 0 )
					{
						$xmlOutputEntry->set_attribute('entrycaption', fileconv( $entrycaption ) );
					}
					if ( strlen( $entrydescription ) > 0 )
					{
						$xmlOutputEntry->set_attribute('entrydescription', fileconv( $entrydescription ) );
					}
					$xmlOutputEntry->set_attribute('entryname', $entryname );
					$xmlOutputRoot->append_child( $xmlOutputEntry );
				}
			}
			
			// HTML生成終了
			
			// XSL変換処理
			//$strFormHTML = ADXslt::process( $xmlDocOutput, dirname(__FILE__) . "/entryeditor.xsl" );
			$strFormHTML = CommonActionUtil::XsltEntryEditor( $xmlDocOutput );
			
			/*
			$xslt = domxml_xslt_stylesheet_file( dirname(__FILE__) . "/entryeditor.xsl" );
			$result = $xslt->process( $xmlDocOutput );
			$strFormHTML = $result->dump_mem( false, 'EUC-JP' );
			*/
			
			// 文字化け不具合回避コード。2006.07.06 追加
			$strFormHTML = replaceMetaTag( $strFormHTML, 'utf-8', gCMSDInternalEncoding() );
			
			return $strFormHTML;
			
		}
	}
	
	
	class EntriesForEditor
	{
		var $entryName = "";	// エントリ名
		var $pageNo = 1;	// ページ番号
		var $maxPageRows = 15;	// １ページ表示件数
		
		var $entryConfig = null;
		
		/* ----------------------------------------------------------------
		 * コンストラクタ。
		 */		
		function EntriesForEditor( $entryName )
		{
			$this->entryName = $entryName;
			
			// 編集画面から戻ってきた場合の画面復帰情報を反映
			$this->maxPageRows = getGETParam( 'maxpagerows', 15 );
			$this->pageNo = getGETParam( 'pageno', 1 );
			
			// Configの読み込み
			$siteConfig = new CMSSiteConfig();
			$entryConfig = $siteConfig->getEntryConfig( $entryName );
			
			$this->entryConfig = $entryConfig;
		}
		
		/* ----------------------------------------------------------------
		 * リクエストの処理。外部から呼び出されるメソッド。
		 */		
		function procRequest()
		{
			// 初回呼び出しならばaction処理を行わない
			if ( ! isset( $_GET['p'] ) )
			{
				// リロード判定：リロードでない場合のみ、各種処理を行う。
				if ( ! ReloadToken::isReload() )
				{
					switch ( getPOSTParam( 'action' ) )
					{
						case 'delete' :
							$itemlist = getPOSTParam(  'items' );
							EntriesForEditor::deleteItems( $this->entryName, $itemlist );
							break;
						
						case 'page' :
							$newPageNo = getPOSTParam( 'pageno' );
							$this->pageNo = $newPageNo;
							break;
						
						case 'rows' :

							$newMaxPageRows = getPOSTParam( 'maxpagerows' );
							$newPageNo = (int) ceil( ((($this->pageNo - 1) * $this->maxPageRows) + 1) / $newMaxPageRows );
							$this->pageNo = $newPageNo;
							$this->maxPageRows = $newMaxPageRows;
							break;
					}
				}
			}
			
			/*
			// Site.Configより指定されたエントリ設定情報を取得する。
			$xmlDocSite = domxml_open_file( App::getSiteConfigFilePath() );
			$xpath = $xmlDocSite->xpath_new_context();
			
			$xmlEntryConfig = $xpath->xpath_eval( "/site/entries/entry[@name='" . $this->entryName . "']" );
			// 2005.7.7 追記
			if ( count($xmlEntryConfig->nodeset) <= 0 )
			{
				cmsd_abort_proc( "site.config.xmlファイルに'{$this->entryName}'のエントリ定義がありません。" );
			}
			//$entryCatpion = htmlconv( $xmlEntryConfigList->nodeset[0]->get_attribute('caption') );
			$entryCatpion = $xmlEntryConfig->nodeset[0]->get_attribute('caption');	// エントリ説明
			$schemaName = $xmlEntryConfig->nodeset[0]->get_attribute('schema');		// スキーマ名
			// 2005.7.7 追加
			if ( $schemaName == '' )
			{
				cmsd_abort_proc( "site.config.xmlファイルの'{$this->entryName}'のエントリ定義にschema属性が指定されていません。" );
			}
			*/
			
			// indexファイルが無い場合、ここで空のindexファイルを生成する。
			if ( ! file_exists( App::createEntryFilePath( $this->entryName, 'index' ) ) )
			{
				// エントリフォルダが書き込み可能かをチェックする。
				$dirpath = App::createEntryDirPath( $this->entryName );
				if ( ! is_writable( $dirpath ) )
				{
					// 書き込み不可のエラーメッセージを出力して終了する。
					cmsd_abort_proc( "エントリフォルダ'$dirpath'が書き込めません。フォルダのパーミッションが707（又は777）以上になっているかを確認してください。" );
				}
				
				// DOMをを生成。
				$xmlDocEntryIndex = domxml_new_doc( '1.0' );
			
				// ルートノード'entry'を生成。
				$xmlDocEntryIndex->add_root( 'entrylist' );
				
				$xmlDocEntryIndex->dump_file( App::createEntryFilePath( $this->entryName, 'index' ), false, true );
				
			}

			// エントリindexのXMLを読み込む。
			$xmlDocEntryIndex = domxml_open_file( App::createEntryFilePath( $this->entryName, 'index' ) );
			$xpath = $xmlDocEntryIndex->xpath_new_context();
			
			/*
			// ページ切替。
			$startEntryPos = ( $this->pageNo - 1 ) * $this->maxPageRows + 1;
			$endEntryPos = $this->pageNo * $this->maxPageRows;
			$pageCond = "";
			*/
			
			// ユーザー権限により、表示範囲を切り替える[2008.04 ユーザー権限]
			$siteConfig = new CMSSiteConfig();
			$entryConfig = $siteConfig->getEntryConfig( $this->entryName );
			$authedUser = LoginManager::GetAuthedUser();
			$authedUserId = $authedUser->getUserId();
			
			$mineOk = $entryConfig->canAction( $authedUser, 'list', 'mine' );		// 自分のエントリを一覧表示可能か？
			$othersOk = $entryConfig->canAction( $authedUser, 'list', 'others' );	// 他人のエントリを一覧表示可能か？
			
			if ( $mineOk && $othersOk )
			{
				// 全てのエントリを表示する
				$xmlEntryList = $xpath->xpath_eval( "/entrylist/entry" );
			}
			elseif ( $mineOk && !$othersOk )
			{
				// 自分のエントリのみ表示する
				$xmlEntryList = $xpath->xpath_eval( "/entrylist/entry[@author='$authedUserId']" );
			}
			elseif( !$mineOk && $othersOk )
			{
				// 他人のエントリのみ表示する
				$xmlEntryList = $xpath->xpath_eval( "/entrylist/entry[@author!='$authedUserId' or not(@author)]" );
			}
			
			// エントリIDが指定されていた場合、該当するエントリIDを含むページ番号を探す。
			$targetEntryId = getGETParam( 'entryid' );
			if ( strlen( $targetEntryId ) > 0 )
			{
				$targetEntryPosition = 0;
				foreach ( cmsd_get_nodeset( $xmlEntryList ) as $entryIndex=>$xmlEntry )
				{
					$entryId = $xmlEntry->get_attribute( 'id' );
					if ( $entryId === $targetEntryId )
					{
						$targetEntryPosition = $entryIndex + 1;
						break;
					}
				}
				
				if ( $targetEntryPosition > 0 )
				{
					$this->pageNo = (int) ceil( $targetEntryPosition / $this->maxPageRows );
				}
			}
			
			// リロードトークンを新しく発行する。
			ReloadToken::newToken();
			
			// 現在の内容で表示処理を行う。
			print( $this->renderHTMLXML( cmsd_get_nodeset( $xmlEntryList ) ) );	
			
		}
		
				
		/* ----------------------------------------------------------------
		 * [クラスメソッド]
		 * 指定したエントリID群を削除する。
		 * $entryName : エントリ名。
		 * $itemlist : エントリIDの配列。
		 */
		function DeleteItems( $entryName, $itemlist )
		{
			if ( ! empty( $itemlist ) )
			{
				$xmlDocEntryIndex = domxml_open_file( App::createEntryFilePath( $entryName, 'index' ) );
				$xpathEntryIndex = $xmlDocEntryIndex->xpath_new_context();
				
				// 削除権限チェック（実際に削除を行う前に一旦全部チェックする）[2008.04 ユーザー権限]
				$siteConfig = new CMSSiteConfig();
				$entryConfig = $siteConfig->getEntryConfig( $entryName );
				$authedUser = LoginManager::GetAuthedUser();
				$authedUserId = $authedUser->getUserId();
				
				$deleteMineOk = $entryConfig->canAction( $authedUser, 'delete', 'mine' );
				$deleteOthersOk = $entryConfig->canAction( $authedUser, 'delete', 'others' );
				
				foreach ( $itemlist as $deleteitem )
				{
					// Indexから該当エントリの情報を取得
					$xmlEntryRefArr = $xpathEntryIndex->xpath_eval( "/entrylist/entry[@href='" . App::createEntryFileName( $entryName, $deleteitem ) . "']" );
					
					if ( count( cmsd_get_nodeset( $xmlEntryRefArr ) ) > 0 ) 
					{
						// Index内に指定エントリが存在したなら、権限チェックを行う。
						$xmlEntryRef = $xmlEntryRefArr->nodeset[0];
						$author = $xmlEntryRef->get_attribute('author');
						
						if ( ( ( $authedUserId === $author ) && ! $deleteMineOk ) ||
							 ( ( $authedUserId !== $author ) && ! $deleteOthersOk ) )
						{
							cmsd_abort_proc( "No permisson to delete the entry : $entryName - $deleteitem." );
						}
						
					}
					
				}
				
				// 実際の削除処理
				foreach ( $itemlist as $deleteitem )
				{
					// 対象エントリが存在する場合のみ処理を実行する。
					if ( file_exists( App::createEntryFilePath( $entryName, $deleteitem ) ) )
					{
					
						// Indexから削除する。
						$xmlEntryRefArr = $xpathEntryIndex->xpath_eval( "/entrylist/entry[@href='" . App::createEntryFileName( $entryName, $deleteitem ) . "']" );
						
						if ( count( cmsd_get_nodeset( $xmlEntryRefArr ) ) > 0 ) 
						{
							// Index内に指定エントリが存在したなら、削除を行う。
							$xmlEntryRefArr->nodeset[0]->unlink_node();
						}
								
						// エントリファイルを削除する。
						unlink( App::createEntryFilePath( $entryName, $deleteitem ) );
						
						// エントリに関連するファイル群を削除する。
						$deletefiles = glob( App::createEntryDirPath( $entryName ) . $entryName . '.' . $deleteitem . '.*' );
						if ( $deletefiles )
						{
							foreach ( $deletefiles as $filepath )
							{
								unlink( $filepath );
							}
						}
					}
				}
				
				ADDom::SaveXmlToFile( $xmlDocEntryIndex, App::createEntryFilePath( $entryName, 'index' ), 'UTF-8' );
				//$xmlDocEntryIndex->dump_file( App::createEntryFilePath( $entryName, 'index' ), false, true );
				
				// 全キャッシュファイルの削除 2009.01.30 追加 j.shinde
				ADPageCache::deleteAllCaches();

			}
		}
		
		/* ----------------------------------------------------------------
		 * 画面の描画。XMLで画面情報を生成し、XSLTを使って整形結果を出力する。
		 */
		function renderHTMLXML( $xmlEntryList )
		{
		
			// スキーマファイルが存在するかまたは読み込み可能かをチェックする。
			$schemaFilePath = App::createSchemaFilePath( $this->entryConfig->schemaName );
			if ( ! is_readable( $schemaFilePath ) )
			{
				cmsd_abort_proc( "スキーマファイル '$schemaFilePath' が存在しないか読み込み不可になっています。" );
			}

			// スキーマから「エントリ見出し」として扱う項目名を取得する。
			$xmlDocSchema = domxml_open_file( $schemaFilePath );
			
			// XMLの読み込みに成功したか？
			if ( empty( $xmlDocSchema ) )
			{
				cmsd_abort_proc( "スキーマファイル '$schemaFilePath' の読み込みに失敗しました。ファイルが正しいXML形式ではない可能性があります。" );
			}
			
			// 一覧の見出し用項目名を取得する
			$xpathSchema = $xmlDocSchema->xpath_new_context();
			$titleDataNameList = $xpathSchema->xpath_eval( "/schema/@title" );
			$titleDataName = '';	// 初期化
			if ( is_object($titleDataNameList) && !empty($titleDataNameList->nodeset) )
			{
				$titleDataName = $titleDataNameList->nodeset[0]->get_content();
			}
			// - 見出し用項目名が設定されていない場合、スキーマ内の先頭項目を見出し用項目とする。
			if ( strlen( $titleDataName ) <= 0 )
			{
				$xmlSchemaItemArr = $xpathSchema->xpath_eval( "/schema/data[@type!='separator'][1]" );	// 2006.2.8 変更 セパレータ項目追加対応
				$titleDataName = $xmlSchemaItemArr->nodeset[0]->get_attribute('name');
			}
			
			// ページ切替用の情報を生成
			$maxPageRows = $this->maxPageRows;
			$pageNo = $this->pageNo;
			$startEntryPos = ( $pageNo - 1 ) * $maxPageRows;
			$endEntryPos = $pageNo * $maxPageRows - 1;
			$allEntryCount = count( $xmlEntryList ); 
			$endPageNo = (int) ceil( $allEntryCount / $maxPageRows ); //(int)( (int)$allEntryCount / (int)$maxPageRows ) + (( ($allEntryCount % $maxPageRows > 0 )) ? 1 : 0 );
			
			// ユーザー権限により、新規ボタン、削除?ボックスを非表示にする[2008.04 ユーザー権限]
			$siteConfig = new CMSSiteConfig();
			$entryConfig = $siteConfig->getEntryConfig( $this->entryName );
			$authedUser = LoginManager::GetAuthedUser();
			$authedUserId = $authedUser->getUserId();
			
			$addOk = $entryConfig->canAction( $authedUser, 'add', 'mine' );					// 自分のエントリを新規追加可能か？
			$deleteMineOk = $entryConfig->canAction( $authedUser, 'delete', 'mine' );		// 自分のエントリを削除可能か？
			$deleteOthersOk = $entryConfig->canAction( $authedUser, 'delete', 'others' );	// 他人のエントリを削除可能か？
			$editMineOk = $entryConfig->canAction( $authedUser, 'edit', 'mine' );			// 自分のエントリを編集可能か？
			$editOthersOk = $entryConfig->canAction( $authedUser, 'edit', 'others' );		// 他人のエントリを編集可能か？
			
			// HTML生成開始	
			// DOMをを生成。
			// ルートノード'entryeditor'を生成。
			$xmlDocOutput = domxml_new_doc( '1.0' );
			$xmlOutputRoot = $xmlDocOutput->add_root( 'entries' );
			$xmlOutputRoot->set_attribute('entrycaption', fileconv( $this->entryConfig->caption ) );
			$xmlOutputRoot->set_attribute('entryname', $this->entryName );
			$xmlOutputRoot->set_attribute('publishcontrol', $this->entryConfig->publishControl );
			
			// エントリ一覧表示用URLが指定されていたら、出力する。
			if ( ! empty($this->entryConfig->listPageUrl) )
			{
				$xmlOutputRoot->set_attribute('listpageurl', $this->entryConfig->listPageUrl );
			}
			
			// 新規追加権限を持っているかどうか。[2008.04 ユーザー権限]
			if ( $addOk )
			{
				$xmlOutputRoot->set_attribute( 'addok', 'yes' );	// 権限無しの場合は属性自体無し
			}
			
			// 自分又は他人のエントリを削除可能か。どちらも削除できないなら、削除ボタンそのものを非表示にする。その為の情報。[2016.08 ユーザー権限 削除ボタン非表示対応]
			if ( $deleteMineOk or $deleteOthersOk )
			{
				$xmlOutputRoot->set_attribute( 'deleteok', 'yes' );	// 権限無しの場合は属性自体無し
			}
			
			// indexファイルから各ファイル名を取得し、該当ファイルのDOMを開いて要約を表示する。
			foreach( $xmlEntryList as $entryIndex=>$xmlEntryRef )
			{
				// 現在のページ内ならば出力する
				if ( $entryIndex >= $startEntryPos and $entryIndex <= $endEntryPos )
				{
				
					// エントリのルート要素から各値を取得
					$filename = $xmlEntryRef->get_attribute( 'href' );
					$xmlDocEntry = domxml_open_file( App::createEntryDirPath( $this->entryName ) . $filename );
					$xmlEntry = $xmlDocEntry->document_element();
					$xpathEntry = $xmlDocEntry->xpath_new_context();
				
					$id = $xmlEntry->get_attribute('id');
					$date = $xmlEntry->get_attribute('date');
					//$visible = $xmlEntry->get_attribute('visible') ? '公開' : '非公開';
					$author = $xmlEntry->get_attribute( 'author' );
					
					
					// 見出しを生成する。見出し項目名が複数指定されている場合もあるので、","でsplitしてループを回す。
					$title = "";
					$titleDataNameArr = explode( ",", $titleDataName );
					foreach ( $titleDataNameArr as $dataName )
					{
						// その項目をエントリから取得する。
						$xmlEntryChildList = $xpathEntry->xpath_eval("/entry/" . $dataName );
						if ( cmsd_get_nodeset( $xmlEntryChildList ) )
						{
							if ( strlen( $title ) > 0 ){
								$title .= " | ";
							}
							
							$xmlSchemaItemArr = $xpathSchema->xpath_eval( "/schema/data[@name='{$dataName}']" );
							$dataType = $xmlSchemaItemArr->nodeset[0]->get_attribute('type');
							
							switch ( $dataType )
							{
								// ファイルの場合、ファイル名を表示
								case 'img':
								case 'file':
								
									$title .= file2code( $xmlEntryChildList->nodeset[0]->get_attribute('org') );
									break;
								
								// メニューの場合、選択肢キャプションを表示
								case 'menu':
									// エントリデータにあるのはmenuId。ここからキャプション（ラベル）を取得する必要がある。
									$menuId = $xmlEntryChildList->nodeset[0]->get_content();
									
									// メニュー項目が外部データを参照しているかどうか？（2015/1/20対か）
									$dataSource = $entryConfig->getDatasource( $dataName );
									if (is_null($dataSource)) {
										// 外部データ無し。スキーマからメニュー項目ラベルを取得する。
										$xmlSchemaMenuItemArr = $xpathSchema->xpath_eval( "/schema/data[@name='{$dataName}']/menuitem[@id='{$menuId}']" );
										
										if ( !empty($xmlSchemaMenuItemArr->nodeset) )
										{
											$title .= file2code( $xmlSchemaMenuItemArr->nodeset[0]->get_content() );
										}
										else
										{
											$title .= "(不明[id:{$menuId}])";
										}
										
									}
									else {
										// 外部データ有り。データソースからメニュー項目ラベルを取得する。
										if ( array_key_exists( $menuId, $dataSource->menuItems ) ) {
											$title .= $dataSource->menuItems[$menuId];
										}
										else
										{
											$title .= "(不明b[id:{$menuId}])";
										}
									}
									
									
									break;
								
								// それ以外は内容をそのまま
								default:
									$title .=  file2code( $xmlEntryChildList->nodeset[0]->get_content() );
									break;
							}
							
						}
					}
					if ( mb_strlen( $title ) > 50 )
					{
						$title = mb_substr( $title, 0, 50 ) . ".."; // 先頭から50文字分を取り出す。
					}
					
					/*
					// エントリ見出し項目名がスキーマに指定されているか？
					if ( $titleDataName )
					{
						// 指定されていれば、その項目をエントリから取得する。
						$xmlEntryChildList = $xpathEntry->xpath_eval("/entry/" . $titleDataName );
					}
					else
					{
						// エントリ見出し項目名スキーマに指定されていなければ、
						// エントリの最初の子要素のtext()を取得して「要約」とする。
						$xmlEntryChildList = $xpathEntry->xpath_eval("/entry/*");
					}
					
					// 子要素から値を取り出し、先頭から何文字分かを取り出す。
					if ( $xmlEntryChildList->nodeset )
					{
						$xmlEntryChild = $xmlEntryChildList->nodeset[0];
						$title = file2code( $xmlEntryChild->get_content() );
						if ( mb_strlen( $title ) > 50 )
						{
							$title = mb_substr( $title, 0, 50 ) . "..";	// 先頭から50文字分を取り出す。
						}
					}
					*/
					
					// 公開設定を取得する。
					$publishState = CMSEntry::GetEntryPublishState( $xmlEntry );
					if ( $this->entryConfig->publishControl === 'none' )
					{
						$publishState = 'showall';
					}
				
					// 出力用DOMにエントリ１件分の情報を書き出す。
					$xmlOutputEntry = $xmlDocOutput->create_element('entry');
					$xmlOutputEntry->set_attribute('id', $id );
					$xmlOutputEntry->set_attribute('date', $date );
					$xmlOutputEntry->set_content( fileconv( $title ) );
					$xmlOutputEntry->set_attribute('publishstate', $publishState );
					
					// 削除権限があるかどうか？[2008.04 ユーザー権限]
					//	- 自エントリ且つ自エントリ削除Ok 又は 他エントリ且つ他エントリ削除Okならば、削除可能。
					if ( ( ( $authedUserId === $author ) && $deleteMineOk ) || ( ( $authedUserId !== $author ) && $deleteOthersOk ) )
					{
						$xmlOutputEntry->set_attribute('deleteok', 'yes' );	// 権限無しの場合は属性自体出力しない。
					}
					
					// 編集権限あるかどうか？
					//	- 自エントリ且つ自エントリ編集Ok 又は 他エントリ且つ他エントリ編集Okならば、編集可能。
					if ( ( ( $authedUserId === $author ) && $editMineOk ) || ( ( $authedUserId !== $author ) && $editOthersOk ) )
					{
						$xmlOutputEntry->set_attribute('editok', 'yes' );	// 権限無しの場合は属性自体出力しない。
					}
					
					$xmlOutputRoot->append_child( $xmlOutputEntry );
					
				}
			}
			
			if ( true )
			{
				// navi要素を生成
				$xmlNavi = $xmlDocOutput->create_element('navi');
				$xmlNavi->set_attribute('position', $pageNo );
				$xmlNavi->set_attribute('allcount', $allEntryCount );
				$xmlNavi->set_attribute('maxpage', $endPageNo );
				$xmlNavi->set_attribute('maxpagerows', $maxPageRows );
				
				// 前ページが存在すれば、prev要素を追加
				if ( $this->pageNo > 1	)
				{
					$xmlPrev = $xmlDocOutput->create_element('prev');
					$xmlPrev->set_attribute('id', $pageNo - 1 );
					$xmlNavi->append_child( $xmlPrev );
				}
				
				// 存在するページ数分、page要素を追加
				for ( $index = 1; $index <= $endPageNo; $index++ )
				{
					$xmlPage = $xmlDocOutput->create_element('page');
					$xmlPage->set_attribute('id', $index );
					$xmlNavi->append_child( $xmlPage );
				}
				
				// 次ページが存在すれば、next要素を追加
				if ( $allEntryCount > $maxPageRows * $pageNo )
				{
					$xmlNext = $xmlDocOutput->create_element('next');
					$xmlNext->set_attribute('id', $pageNo + 1 );
					$xmlNavi->append_child( $xmlNext );
				}
				
				$xmlOutputRoot->append_child( $xmlNavi );			
			}
			
			// ReloadTokenの出力
			ReloadToken::embedToXmlDoc( $xmlDocOutput );
			
			// HTML生成終了
			// XSL変換処理
			//$strFormHTML = ADXslt::process( $xmlDocOutput, dirname(__FILE__) . "/entryeditor.xsl" );
			$strFormHTML = CommonActionUtil::XsltEntryEditor( $xmlDocOutput );
			/*
			$xslt = domxml_xslt_stylesheet_file( dirname(__FILE__) . "/entryeditor.xsl" );
			$result = $xslt->process( $xmlDocOutput );
			$strFormHTML = $result->dump_mem( false, 'EUC-JP' );
			*/
			
			// 文字化け不具合回避コード。2006.07.06 追加
			$strFormHTML = replaceMetaTag( $strFormHTML, 'utf-8', gCMSDInternalEncoding() );
			
			return $strFormHTML;			
			
		}

		
	}
	
	
	/* --------------------------------------------------------------------
	 * エントリ編集ツール。
	 * エントリを編集する。
	 */
	class EntryEditor
	{
		var $currentEntry = "/entry"; 	// エントリの現在の編集階層位置
		var $currentSchema = "/schema";	// スキーマの現在の編集階層位置
		//var $isNew = True;	  // 新規エントリか？
		var $entryName = "";	// エントリ名
		var $entryId = "";		// エントリID
		var $schemaName = "";	// スキーマ名
		
		var $sortOrder = 'desc';	// ソート並び順
		var $sortKey = '@date';	// ソートキー
		var $groupKeys = NULL;	// グループキー
		
		var $strAllXml = "";	   // 編集対象のXML文字列
		var $isDirty = false; 	// 編集内容が保存されていないか？
		var $errorMessage = '';	// エラーメッセージ
		var $errorId = '';			// エラーID
		var $normalMessage = ''; // 通常メッセージ
		
		var $selectedListItems = array();	 // リスト項目編集画面にて、選択された項目IDの配列。
		
		var $entries_MaxPageRows = 15;	// エントリ一覧画面の戻り先情報：1ページ表示件数
		var $entries_PageNo = 1; 	// エントリー一覧画面の戻り先情報：ページ番号
		
		/* ----------------------------------------------------------------
		 * コンストラクタ。
		 * $entryName : エントリ名。
		 * $id : エントリID。新規の場合は空白か"new"を指定する。
		 */
		function EntryEditor( $entryName, $id )
		{
			$this->entryName = $entryName;
			
			if ( $id === 'new' )
			{
				// 認証済みユーザーの取得
				$user = LoginManager::GetAuthedUser();
				
				// 仮IDの発行。「'_'+ユーザーID」とする事で、ユーザー間の一時ファイル名の衝突を防ぐ。
				$id = '_' . $user->getUserId();
			}
			
			$this->entryId = $id;
			$this->load( $entryName, $id );
			
			$this->entries_MaxPageRows = getGETParam( 'maxpagerows', 15 );
			$this->entries_PageNo = getGETParam( 'pageno' , 1 );
		}
		
		function isNewEntry( $id = null )
		{
			if ( is_null( $id ) )
			{
				$id = $this->entryId;
			}
			
			// エントリIDの先頭1文字が'_'の場合、新規エントリと判断する。
			return ( substr( $id, 0, 1 ) === '_' );
		}
		
		/* ----------------------------------------------------------------
		 * 編集の為の情報をファイルから読み込む。
		 * $entryName : エントリ名。
		 * $id : エントリID。新規の場合は空白か"new"を指定する。
		 */
		function load( $entryName, $id )
		{
			// Site.Configより指定されたエントリ設定情報を取得する。
			$xmlDocSite = domxml_open_file( App::getSiteConfigFilePath() );
			$xpath = $xmlDocSite->xpath_new_context();
			
			$xmlEntryConfig = $xpath->xpath_eval( "//entry[@name='" . $entryName . "']" );
			// 2005.7.7 追加
			if ( count( cmsd_get_nodeset( $xmlEntryConfig )) <= 0 )
			{
				cmsd_abort_proc( "site.config.xmlファイルに'$entryName'のエントリ定義がありません。" );
			}
			$this->schemaName = $xmlEntryConfig->nodeset[0]->get_attribute( "schema" );
			// 2005.7.7 追加
			if ( $this->schemaName === '' )
			{
				cmsd_abort_proc( "site.config.xmlファイルの'$entryName'のエントリ定義にschema属性が指定されていません。" );
			}
			
			// スキーマ情報を取得する。
			$xmlDocSchema = domxml_open_file( App::createSchemaFilePath( $this->schemaName ) );
			$xpathSchema = $xmlDocSchema->xpath_new_context();

			// 新規エントリならばエントリを作成し、既存エントリならばファイルからロードする。
			if ( $this->isNewEntry( $id ) ) // $id== "new" )
			{
				// 空のエントリを作成する
				$this->createNewEntry();
			}
			else
			{
	 			// エントリのXML情報を文字列として取得し、メンバ変数に保持する。
			   	// ※エントリへの編集はこの文字列に対して行い、saveメソッドにてエントリファイルに書き出す。
		   	 	$xmlDocEntry = domxml_open_file( App::createEntryFilePath( $entryName, $id ) );
				
				// 現在の階層のエントリデータ項目をスキーマと同期させる。
				$this->conformDataToSchema( $xmlDocEntry, $xmlDocSchema );
				
		   		// メンバ変数に文字列として格納する。
				$this->strAllXml = $xmlDocEntry->dump_mem( true, 'UTF-8' );
			}
			
			// スキーマからソートキー情報を取得してメンバに格納しておく。
			
			/// schema要素のsortkey属性、sortorder属性を取得する。
			$xmlSchema = $xmlDocSchema->document_element();
			$sortKey = $xmlSchema->get_attribute( 'sortkey' );
			$sortOrder = $xmlSchema->get_attribute( 'sortorder' );
			
			if ( !empty($sortKey))
			{
				$this->sortKey = $sortKey;
				$this->sortOrder = $sortOrder;
			}
			else
			{
				// sortkey属性が設定されていなければ、旧仕様の方法でソート情報を取得する。
			
				$xmlSchemaSortKeyArr = $xpathSchema->xpath_eval( "/schema//data[@sort!='']" );
				if ( !empty( $xmlSchemaSortKeyArr->nodeset ) )
				{
					$this->sortOrder = $xmlSchemaSortKeyArr->nodeset[0]->get_attribute('sort');
					$this->sortKey = $xmlSchemaSortKeyArr->nodeset[0]->get_attribute('name');
				}
			
			}
			
			// 2014/8/29 added: Abort when the $this->sortKey has a comma.
			if ( strpos( $this->sortKey, ',' ) !== FALSE ) 
			{
				cmsd_abort_proc( "{$this->schemaName}.schema.xml: sortkeyにカンマが含まれています。sortkeyに複数の項目名を指定することはできません。" );
			}
			
			/// sortOrderの省略値は'desc'。
			if ( empty( $this->sortOrder ) )
			{
				$this->sortOrder = 'desc';
			}
			
			// スキーマからグループ情報を取得してメンバに格納しておく。
   			$xmlSchemaGroupKeyArr = $xpathSchema->xpath_eval( "/schema//data[@group='True' or @group='true']" );
			$this->groupKeys = array();
			if ( !empty( $xmlSchemaGroupKeyArr->nodeset ) )
			{
				foreach( cmsd_get_nodeset( $xmlSchemaGroupKeyArr ) as $xmlSchemaGroupKey )
				{
						$this->groupKeys[] = $xmlSchemaGroupKey->get_attribute('name');
				}
			}
			
			/* 2006.08.11 コメントアウト。エントリ保存時に削除している為、ここでもやる必要はないと判断。また、この処理のままだとマルチユーザー時に問題があった。
			// 残っている以前のゴミtmpファイルを削除する。
			$deletefiles = glob( App::createEntryDirPath( $this->entryName) . '*.tmp' );
			if ( $deletefiles )
			{
				foreach ( $deletefiles as $tmpfilePath )
				{
					unlink($tmpfilePath);
				}
			}
			*/

		}
		
		function createNewEntry()
		{
			// DOMをを生成。
			$xmlDocNewEntry = domxml_new_doc( '1.0' );
			
			// ルートノード'entry'を生成。
			$xmlEntryRoot = $xmlDocNewEntry->add_root( 'entry' );
			
 		  	// 現在のスキーマDOMを取得
			$xmlDocSchema = domxml_open_file( App::createSchemaFilePath( $this->schemaName ) );
			$xpathSchema = $xmlDocSchema->xpath_new_context();
			$xmlSchemaDataList = $xpathSchema->xpath_eval( "/schema/data" );
			
			// ファイルIDを初期化
			$fileid = 0; // 0は未設定であることを表す初期値。
			
			foreach( $xmlSchemaDataList->nodeset as $xmlSchemaData )
			{
				// 各要素毎にノードを新規作成する。
				$item = EntryItemFactory::createEntryItem( $xmlSchemaData, $this->entryName, $this->entryId	);
				$newNode = $item->createNew( $xmlDocNewEntry );
				
				// newNodeがnullの場合、ノードの追加は不要なのでスキップする。
				if ( is_null( $newNode ) )	
				{	
					continue;
				}
				
				// imgとfile属性のみ、fileidを採番する。
				$type = $xmlSchemaData->get_attribute('type');
				if ( $type === 'img' or $type === 'file' )
				{
					$fileid = (int)$fileid + 1;
					$newNode->set_attribute( 'fileid', sprintf( "%08d", $fileid ) );	
				}
				
				// 新規作成したノードを子として追加する。
				$xmlEntryRoot->append_child( $newNode );				
				
			}
			
			// エントリ更新日時に現在日時を設定
			/// エントリ更新日時用のスキーマ項目を生成する。
			$xmlSchemaDataEntryDate = $this->createSchemaDataForEntryDate( $xmlDocSchema );
			
			/// エントリ更新日時を更新する。
			$item = EntryItemFactory::createEntryItem( $xmlSchemaDataEntryDate, $this->entryName, $this->entryId	);
			$item->createEntryDate( $xmlEntryRoot );
			
			// 自ユーザー名を設定[2008.04 ユーザー権限]
			$user = LoginManager::GetAuthedUser();
			$xmlEntryRoot->set_attribute( 'author', $user->getUserId() );

			$this->strAllXml = $xmlDocNewEntry->dump_mem( true, 'UTF-8' );
			
		}
		
		/* ----------------------------------------------------------------
		 * 現在の階層のエントリデータ項目を、スキーマと同期させる。
		 */
		function conformDataToSchema( &$xmlDocEntry, &$xmlDocSchema )
		{
			//print "conformDataToSchema<br/>\n";
			// エントリXMLのDOMを作成する。
			//$xmlDocEntry = domxml_open_mem( $this->strAllXml );
			$xpathEntry = $xmlDocEntry->xpath_new_context();
			
			// 現在の階層のエントリを取得する。
			$xmlParentDataArr = $xpathEntry->xpath_eval( $this->currentEntry );
			$xmlParentData = $xmlParentDataArr->nodeset[0];
			
			// 現在の階層のスキーマの各"data"要素を取得する。
			//$xmlDocSchema = domxml_open_file( App::createSchemaFilePath( $this->schemaName ) );
			$xpathSchema = $xmlDocSchema->xpath_new_context();
			$xmlSchemaDataArr = $xpathSchema->xpath_eval( $this->currentSchema . "/data" );	// 2006.02.08 メモ：@type!='separator'で絞った方が効率が良いが、separatorを特別扱いしたくないのでこのままにした。
			
			// 現在の階層のデータを、スキーマと同期させる。（項目が存在しない場合、空項目を追加する）
			
			$fileid = 0; // ファイルID採番用変数。0は未設定を表す初期値。
			
			// listitem要素に空の中身を設定する。
			if ( count( cmsd_get_nodeset( $xmlSchemaDataArr )) > 0 )
			{
				foreach( cmsd_get_nodeset($xmlSchemaDataArr) as $xmlSchemaData )
				{
					// EntryItemオブジェクトを生成する。
					$item = EntryItemFactory::createEntryItem( $xmlSchemaData, $this->entryName, $this->entryId );
					
					// セパレータの場合は何もしない。2006.02.08 セパレータ項目対応。
					if ( $item->getType() != 'separator' )
					{
						// この項目がエントリに存在するか？
						/// エントリXMLを取得。
						$xmlEntryDataArr = $xpathEntry->xpath_eval( $this->currentEntry . '/' . $item->getName() );
						//print "conformDataToSchema : Checked EntryData, " . $item->getName() . ", " . get_class($xmlEntryDataArr->nodeset[0]) . "<br/>\n";
						
						if ( count( cmsd_get_nodeset($xmlEntryDataArr) ) == 0 )
						{
							//print "conformDataToSchema : Added EntryData</ br>";
							// 存在しない場合、ノードを新規生成する。
							$newNode = $item->createNew( $xmlDocEntry, false );
						
							// imgとfile属性のみ、fileidを採番する。
							$type = $xmlSchemaData->get_attribute('type');
							if ( $type === 'img' or $type === 'file' )
							{
								if ( (int)$fileid <= 0 )
								{
									// fileidを採番する。同一エントリ中の最大IDに+1する。
									$fileid = EntryUtil::getMaxFileId( $xpathEntry ) + 1;
								}
								else
								{
									$fileid = (int)$fileid + 1;
								}					
								$newNode->set_attribute( 'fileid', sprintf( "%08d", $fileid ) );	
							}
					
							// リスト項目に子ノードとして追加する。					
							$xmlParentData->append_child( $newNode );
						
						}
					}
				
				}
			}
		}
		
		
		function procRequest()
		{
			// エラーメッセージを初期化する。
			$this->errorMessage = '';
			$this->errorId = '';
			
			// 選択リスト項目番号配列を初期化する。
			$this->selectedListItems = array();
			
			// 通常メッセージを初期化する。
			$this->normalMessage = '';
			
			// 認証済みユーザーの取得
			$user = LoginManager::GetAuthedUser();
			
			// forceLock依頼があった場合、エントリロックを解除する。
			if ( getPOSTParam( 'action' ) === 'forceLock' )
			{
				EditingEntryLock::ForceLock( $this->entryName, $this->entryId, $user->getUserId() );
			}
			
			// エントリロックを取得する（新規エントリ以外の場合のみ）
			if ( $this->entryId != 'new' )
			{
				
				// ロックを取得する。
				$lock = EditingEntryLock::CatchLock( $this->entryName, $this->entryId, $user->getUserId() );
				if ( ! $lock->isOwner( $user->getUserId() ) )
				{
					// ロックを取得できなかった場合は、エラー表示をする。
					$ownerUserId = $lock->ownerUserId;
					$lastUpdate = date( 'Y-m-d H:i:s', $lock->lastUpdate );
					$this->errorMessage = "他のユーザー($ownerUserId)が編集中です。最終操作時刻は $lastUpdate です。編集権を奪いますか？";
					$this->errorId = "alreadyLocked";
					
				}
				
			}
			
			// 編集権限があるか？[2008.04 ユーザー権限]
			$siteConfig = new CMSSiteConfig();
			$entryConfig = $siteConfig->getEntryConfig( $this->entryName );
			
			$xmlDocEntry = domxml_open_mem( $this->strAllXml );
			$xmlEntryRoot = $xmlDocEntry->document_element();
			$author = $xmlEntryRoot->get_attribute( 'author' );
								
			// 管理者ならば無条件で権限有り。
			if ( ! $user->isAdministrator() )
			{
				if ( $this->isNewEntry() )
				{
					// 新規モード時
					if ( ! $entryConfig->canAction( $user, 'add', 'mine' ) )
					{
						// 新規追加権限無ければエラー。
						$this->errorMessage = "新規エントリの追加権限がありません。";
						$this->errorId = "noPermissionForAdding";
					}
				}
				else
				{

				
					// 編集モード時
					if ( ( ( $author === $user->getUserId() ) && ! $entryConfig->canAction( $user, 'edit', 'mine'	) ) ||
						 ( ( $author !== $user->getUserId() ) && ! $entryConfig->canAction( $user, 'edit', 'others' ) ) )
					{
						// 自エントリ且つ自エントリ編集権限が無い、又は他エントリ且つ他エントリ編集権限が無い場合はエラー。
						$this->errorMessage = 'このエントリの編集権限がありません。';
						$this->errorId = 'noPermissionForEditting';
					}
				}
			}
						
			// 初回呼び出しか？(且つ、エラーが発生していないか？)
			if ( ! isset( $_GET['p'] ) && $this->errorId === '' )
			{
				// 初回呼び出しでないなら、リロード判定を行う。
				if ( ! ReloadToken::isReload() )
				{
					// リロードされていないなら、各種処理行う。
					
					switch ( $_POST['action'] )
					{
						case 'move' :
							$this->update();
							if ( $this->errorMessage === '' ) // エラーでなければ移動
							{
								$this->moveLevel( $_POST['nextlevel'] );
							}
							break;

						case 'update' :
							$this->update();
							$this->isDirty = true;
							break;
														
						case 'save' :
							$this->update();
							$this->isDirty = true;
							if ( $this->errorMessage === '' ) // エラーでなければ保存
							{
								$this->save();
								$this->isDirty = false;
							}
							break;
						
						case 'newitem' :
							$this->newItem();
							$this->isDirty = true;
							break;
						
						case 'delitems' :
							$itemlist = $_POST['items'];
							$this->deleteItems( $itemlist );
							$this->isDirty = true;
							break;
						
						case 'moveitems' :
							$itemlist = $_POST['items'];
							$moveto = $_POST['moveto'];
							$this->moveItems( $itemlist, $moveto );
							$this->isDirty = true;
							$this->selectedListItems = $itemlist;
							break;
 					}
					
					//print "Current : " . ReloadToken::getCurrentToken() . "<br>\n";
					//print "Request : " . ReloadToken::getRequestToken() . "<br>\n";
					
				}
				else
				{
					// [2015.9.4]ファイルアップロードでpost_max_sizeを超えた場合、POSTやFILESの中身が空っぽになってしまう。この為、リロードトークンが取得できない。その場合はエラーメッセージを表示する。
					if ( empty( $_POST ) && empty( $_FILES ) && isset( $_SERVER['REQUEST_METHOD'] ) && strtolower( $_SERVER['REQUEST_METHOD'] ) == 'post' ) 
					{
						$this->errorMessage = "一度にアップロードするファイルのサイズがサーバの上限値を超えています。ファイルサイズを小さくするか、サーバ管理者に連絡して上限値を変更してください。";
					}
				
					// 不正リロード時は特にエラーメッセージを出さない。
					
					//print "Reloaded!!<br>\n";
					//print "Current : " . ReloadToken::getCurrentToken() . "<br>\n";
					//print "Request : " . ReloadToken::getRequestToken() . "<br>\n";
					
				}
			}
			
			// リロードトークンを新しく発行する。
			ReloadToken::newToken();		
			
			// 現在の内容で表示処理を行う。 		  
			//print( $this->renderFormHTML() );
			print( $this->renderFormXML() );
	
						
		}
		
		
		function moveLevel( $dataName )
		{
			// 現在のスキーマDOMを取得
			$xmlDocSchema = domxml_open_file( App::createSchemaFilePath( $this->schemaName ) );
			$xpathSchema = $xmlDocSchema->xpath_new_context();
			$xmlSchemaDataList = $xpathSchema->xpath_eval( $this->currentSchema );
			$xmlSchemaData = $xmlSchemaDataList->nodeset[0];
		
		
			// $dataName が".."だったら１つ上の階層へ移動、
			// "^"だったら一番上の階層へ移動、
			// それ以外なら指定された下位の階層へ移動。
			
			if ( $dataName === ".." )
			{
				// 現在のxpathから、1階層("/"で区切られた部分）分、削除する。
				$pos = strrpos( $this->currentEntry, '/' );
				$this->currentEntry = 
					substr( $this->currentEntry, 0, $pos ); //$pos, strlen( $this->currentEntry ) - $pos );
				
				$pos = strrpos( $this->currentSchema, '/' );
				$this->currentSchema = 
					substr( $this->currentSchema, 0, $pos); // $pos, strlen( $this->currentSchema ) - $pos );
				
			}
			else if ( substr( $dataName, 0, 1 ) === "^" ) // 先頭の文字が"^"だったら
			{
				// "^nn" のパターンであることが前提
				
				// 指定回数分、上の階層へ移動する。
				$iLevel = substr( $dataName, 1 );
				if ( ! is_numeric( $iLevel ) )
				{
					$iLevel = 1;
				}
				
				$schemaXPath = $this->currentSchema;
				$entryXPath = $this->currentEntry;
				for ( $iCount = 0; $iCount < $iLevel; $iCount ++ )
				{
					$schemaXPath = $this->getUpperLevelXPathStr( $schemaXPath );
					$entryXPath = $this->getUpperLevelXPathStr( $entryXPath );
					
					// もし、"/entry"に到達したら、途中でもbreak;
					if ( $entryXPath === '/entry' )
					{
						break;
					}
				}
				
				$this->currentSchema = $schemaXPath;
				$this->currentEntry = $entryXPath;
				
			}
			else  // 下の階層へ移動
			{
				//print "moveLevel, ";
				//print $xmlSchemaData->get_attribute( 'name' ) . ", ";
				if ( $xmlSchemaData->get_attribute( 'type' ) === 'list' )
				{
					// 現階層がリストの場合、エントリのxpathにidによる絞込みを付加する。
					$this->currentEntry .= "/listitem[@id='$dataName']";

					// スキーマのxpathに、nameによる絞込みを付加する。
					$this->currentSchema .= "/listitem";
					
				}
				else
				{
					// 現階層がリスト以外の場合、エントリのxpathにタグ名による絞込みを付加する。
					$this->currentEntry .= "/$dataName";
	
					// スキーマのxpathに、nameによる絞込みを付加する。
					$this->currentSchema .= "/data[@name='$dataName']";
					
				}
				
				// 移動先の階層をスキーマと同期させる。
				$xmlDocEntry = domxml_open_mem( $this->strAllXml );
				$this->conformDataToSchema( $xmlDocEntry, $xmlDocSchema );
				// DOMツリーを文字列化してメンバに戻す。
				$this->strAllXml = $xmlDocEntry->dump_mem( true, 'UTF-8' );
			}
			
		}
		
		/* ----------------------------------------------------------------
		 * リストに新しい項目を追加する。
		 * 現在の階層の型が"list"なら、新規要素を追加してIDを返却する。
		 */
		function newItem()
		{
			// 新規項目のスキーマを取得する。
			$xmlDocSchema = domxml_open_file( App::createSchemaFilePath( $this->schemaName ) );
			$xpathSchema = $xmlDocSchema->xpath_new_context();
			$xmlSchemaDataArr = $xpathSchema->xpath_eval( $this->currentSchema . "/listitem/data" );
			
			// エントリXMLのDOMを作成する。
			$xmlDocEntry = domxml_open_mem( $this->strAllXml );
			$xpathEntry = $xmlDocEntry->xpath_new_context();
				
			// 新規項目のIDを決定する。既存IDの最大値＋１とする。
			$xmlEntryListItemArr = $xpathEntry->xpath_eval( $this->currentEntry . "/listitem" );
			$newID = 1; // 初期値は1
			foreach( cmsd_get_nodeset( $xmlEntryListItemArr ) as $xmlEntryListItem )
			{
				$itemid = $xmlEntryListItem->get_attribute('id');
				if ( (int)$newID <= (int)$itemid )
				{
					// 現在の値以上のIDを発見したら、それよりさらに+1する。
					$newID = (int) $itemid + 1;
				}
			}
			
			//print "newId : " . $newID . "<br>\n";
			
			// 新しいIDでlistitem要素を作成する。
			$xmlEntryNewListItem = $xmlDocEntry->create_element( 'listitem' );
			$xmlEntryNewListItem->set_attribute( 'id', $newID );
			
			$fileid = 0; // 0は未設定を表す初期値。
			
			// listitem要素に空の中身を設定する。
			foreach( cmsd_get_nodeset( $xmlSchemaDataArr ) as $xmlSchemaData )
			{
				// 各要素毎にノードを新規作成する。
				$item = EntryItemFactory::createEntryItem( $xmlSchemaData, $this->entryName, $this->entryId	);
				$newNode = $item->createNew( $xmlDocEntry );
				
				// 戻り値がnullなら何もしない	2006.02.08 追加 セパレータ項目追加対応
				if ( ! is_null( $newNode ) )
				{
				
					// imgとfile属性のみ、fileidを採番する。
					$type = $xmlSchemaData->get_attribute('type');
					if ( $type === 'img' or $type === 'file' )
					{
						if ( (int)$fileid <= 0 )
						{
							// fileidを採番する。同一エントリ中の最大IDに+1する。
							$fileid = EntryUtil::getMaxFileId( $xpathEntry ) + 1;
						}
						else
						{
							$fileid = (int)$fileid + 1;
						}					
						$newNode->set_attribute( 'fileid', sprintf( "%08d", $fileid ) );	
					}
					
					// リスト項目に子ノードとして追加する。					
					$xmlEntryNewListItem->append_child( $newNode );
				}
				
			}
			
			// listitemをDOMツリーに挿入する。
			$xmlEntryCurrentArr = $xpathEntry->xpath_eval( $this->currentEntry );
			$xmlEntryCurrentArr->nodeset[0]->append_child( $xmlEntryNewListItem );
			
			// DOMツリーを文字列化してメンバに戻す。
			$this->strAllXml = $xmlDocEntry->dump_mem( true, 'UTF-8' );
			
			// カレント位置を、今追加したlistitemへ移動する。
			$this->moveLevel( (string)$newID );
			
		}
		
		/* ----------------------------------------------------------------
		 * 選択されたリスト項目をリストから削除する。
		 * 現在の階層の型が"list"なら、選択された要素をリストから削除する。
		 */
		function deleteItems( $itemlist )
		{
			if ( !empty( $itemlist ) )
			{
				// エントリのDOMを生成
				$xmlDocEntry = domxml_open_mem( $this->strAllXml );
				$xpathEntry = $xmlDocEntry->xpath_new_context();
				
				foreach( $itemlist as $itemid )
				{
					$xmlEntryListItemArr = $xpathEntry->xpath_eval( $this->currentEntry . "/listitem[@id='" . $itemid . "']" );	
					if ( !empty( $xmlEntryListItemArr ) )
					{
						$xmlEntryListItemArr->nodeset[0]->unlink_node();
					}
				}
				
				// DOMを文字列化してメンバ変数に戻す。
				$this->strAllXml = $xmlDocEntry->dump_mem( true, 'UTF-8' );
				
			}
		}
		

		/* ----------------------------------------------------------------
		 * 選択されたリスト項目をリスト内で移動する。複数選択の場合、まとめて移動する。移動先ではひと塊になる。
		 * 上へ移動する場合は選択リストの先頭番号-1へ。下へ移動する場合は選択リストの末尾番号+1へ移動する。
		 * $itemlist : 選択番号の配列。
		 * $moveto : 'top'なら先頭へ、'bottom'なら末尾へ、'up'なら１つ上へ、'down'なら１つ下へ移動。
		 */
		function moveItems( $itemlist, $moveto )
		{
			//print $moveto . '<br />';
		
			if ( !empty( $itemlist ) )
			{
				// エントリのDOMを生成
				$xmlDocEntry = domxml_open_mem( $this->strAllXml );
				$xpathEntry = $xmlDocEntry->xpath_new_context();
				
				// リスト項目をDOMから切り離し、別リストに保存する。
				$xmlMoveItemArr = array();
				
				// リスト全体を取得
				$xmlEntryListItemArr = $xpathEntry->xpath_eval( $this->currentEntry . "/listitem" );
				
				foreach ( cmsd_get_nodeset( $xmlEntryListItemArr ) as $itemIndex=>$xmlItem )
				{
					$itemID = $xmlItem->get_attribute('id');
					if ( in_array( $itemID, $itemlist ) )
					{
						
						// 'up'なら先頭選択項目の１つ前、'down'なら末尾選択項目の１つ後を取得しておく。
						if ( $moveto === 'up' and ! isset( $xmlInsertPoint ) )
						{
							if ( $itemIndex === 0 )
							{
								//print '先頭データ<br/>';
								$xmlInsertPoint = 'topnode';	// 後でノードに置き換える。
							}
							else
							{
								$xmlInsertPoint =& $xmlEntryListItemArr->nodeset[$itemIndex-1]; //$xmlItem->previous_sibling();
							}
						}
						else if ( $moveto === 'down' )
						{
							// 最後尾又は最後尾-1ならばnullを、それ以外なら+2の位置を挿入ポイントとする。(insert_beforeを使うため、2つ後ろを取得しておく必要がある）
							if ( $itemIndex < count($xmlEntryListItemArr->nodeset) - 2 )
							{
								$xmlInsertPoint =& $xmlEntryListItemArr->nodeset[$itemIndex+2];
							}
							else
							{
								$xmlInsertPoint = 'bottomnode';    // 後でノードに置き換える。
							}
							/*
							$xmlInsertPoint =& $xmlItem->next_sibling();
							if ( ! is_null( $xmlInsertPoint ) )
							{
								// 最後尾（=null）じゃないなら、もう一つ次を取得しておく（insert_beforeを使うので、2つ後ろを取得しておく必要がある）。
								$xmlInsertPoint =& $xmlInsertPoint->next_sibling();  
							}
							*/
						}
						
						// DOMから削除して別リストに保存。
						$xmlItem->unlink_node();
						$xmlMoveItemArr[] = $xmlItem;
						
					}
				}
				
				
				if ( $moveto === 'top' or $xmlInsertPoint === 'topnode' )
				{
					// 先頭ノードを挿入ポイントに設定。先頭ノードとは、移動リストを差し引いた残りの先頭ノードなので、検索しなければならない。
					$xmlInsertPoint = null;
					foreach ( cmsd_get_nodeset( $xmlEntryListItemArr ) as $itemIndex=>$xmlItem )
					{
						// 移動対象項目では「ない」ノードを発見したら、そこが挿入ポイント。
						$itemID = $xmlItem->get_attribute('id');
						if ( ! in_array( $itemID, $itemlist ) )
						{
							$xmlInsertPoint =& $xmlEntryListItemArr->nodeset[$itemIndex];
							break;
						}
					}
					
					// 検索されなかった場合、全項目移動ということなので、挿入位置を'bottomnode'に変更する。
					if ( is_null( $xmlInsertPoint ) )
					{
						$xmlInsertPoint = 'bottomnode';
					}
				}
				
				if ( $moveto === 'bottom' or $xmlInsertPoint === 'bottomnode' )    // 挿入ポイントが'bottomnode'なのは、'down'指定で末尾選択項目がリストの末尾だった場合。
				{
					// 最後尾にappendする。
					$xmlEntryCurrentArr = $xpathEntry->xpath_eval( $this->currentEntry );
					foreach( $xmlMoveItemArr as $xmlMoveItem )
					{
						$xmlEntryCurrentArr->nodeset[0]->append_child( $xmlMoveItem );
					}
				}
				else
				{
					//print 'insertPoint : ' . $xmlInsertPoint->get_attribute('id') . '<br />';
				
					// 挿入ポイントの前に挿入する。
					$xmlEntryCurrentArr = $xpathEntry->xpath_eval( $this->currentEntry );
					foreach( $xmlMoveItemArr as $xmlMoveItem )
					{
						$xmlEntryCurrentArr->nodeset[0]->insert_before( $xmlMoveItem, $xmlInsertPoint );
					}
				}
				
				// DOMを文字列化してメンバ変数に戻す。
				$this->strAllXml = $xmlDocEntry->dump_mem( true, 'UTF-8' );
				
				//print '<pre>' . $this->strAllXml . '</pre><br />';
				
			}
		}		 
		
		
		/* ----------------------------------------------------------------
		 * 入力データを編集中のデータに反映（ファイルにはまだ反映しない）。
		 */		
		function update()
		{		
			// 現在の階層のスキーマに対応するリクエストの値を、現在のエントリに反映する。
			
			// エントリXMLのDOMを作成する。
			$xmlDocEntry = domxml_open_mem( $this->strAllXml );
			$xpathEntry = $xmlDocEntry->xpath_new_context();
			
			// 現在の階層のスキーマの各"data"要素を取得する。
			$xmlDocSchema = domxml_open_file( App::createSchemaFilePath( $this->schemaName ) );
			$xpathSchema = $xmlDocSchema->xpath_new_context();
			$xmlSchemaDataList = $xpathSchema->xpath_eval( $this->currentSchema . "/data" );
			
			// Configの読み込み
			$siteConfig = new CMSSiteConfig();
			$entryConfig = $siteConfig->getEntryConfig( $this->entryName );
			
			// 現在の階層がトップ階層ならば、エントリ更新日時の項目を更新する
			if ( $this->currentEntry === '/entry' )
			{
				// エントリ更新日時用のスキーマ項目を生成する。
				$xmlSchemaDataEntryDate = $this->createSchemaDataForEntryDate( $xmlDocSchema );
				
				// エントリ更新日時を更新する。
				$item = EntryItemFactory::createEntryItem( $xmlSchemaDataEntryDate, $this->entryName, $this->entryId	);
				$resultOK = $item->update( $xpathEntry, $this->currentEntry );
				if ( ! $resultOK )
				{
					$this->errorMessage = $item->getErrorMessage();
					return;
				}
				
				// その他のヘッダ情報を更新する。
				$item = new HeaderEntryItem( $this->entryName, $this->entryId, $entryConfig->publishControl );
				$resultOK = $item->update( $xpathEntry, $this->currentEntry );
				if ( ! $resultOK )
				{
					$this->errorMessage = $item->getErrorMessage();
					return;
				}
			}
			
			// スキーマのdata要素に対応する入力があれば、エントリに反映する。
			foreach( cmsd_get_nodeset( $xmlSchemaDataList ) as $xmlSchemaData )
			{
				//print( "className = " . get_class( $xmlSchemaData ) . "<br>\n" );
				// データ名をスキーマから取得する。
				$dataname = $xmlSchemaData->get_attribute( 'name' );
				$datatype = $xmlSchemaData->get_attribute( 'type' );
				
				//print( $dataname . " = " . htmlconv( $datavalue ) . "<br>\n" );
				
				$xmlEntryDataList = $xpathEntry->xpath_eval( $this->currentEntry . "/$dataname" );
				if ( !empty( $xmlEntryDataList ) )	// エントリデータに、スキーマ項目に対応するデータがあるか？
				{
					if ( $datatype != 'list' )	// それは、list項目以外か？（list項目なら無視）
					{
						$item = EntryItemFactory::createEntryItem( $xmlSchemaData, $this->entryName, $this->entryId	);
						$resultOK = $item->update( $xpathEntry, $this->currentEntry );
						if ( ! $resultOK )
						{
							$this->errorMessage = $item->getErrorMessage();
						}
					}
				}
			}
			
			$this->strAllXml = $xmlDocEntry->dump_mem( true, 'UTF-8' );
		}
		
		/* ----------------------------------------------------------------
		 * 編集中のデータをファイルに保存。
		 */		
		function save()
		{
			// -- チェック -----------------------------------------------------

			// エントリフォルダが書き込み可能かをチェックする。
			$checkpath = App::createEntryDirPath( $this->entryName );
			if ( ! is_writable( $checkpath ) )
			{
				// 書き込み不可のエラーメッセージを出力して終了する。
				cmsd_abort_proc( "エントリフォルダ'$checkpath'が書き込めません。フォルダのパーミッションが707（又は777）以上になっているかを確認してください。" );
			}
			
			// エントリindexファイルが存在するならば、書き込み可能かをチェックする。
			$checkpath = App::createEntryFilePath( $this->entryName, 'index' );
			if ( file_exists( $checkpath ) )
			{
				if ( ! is_writable( $checkpath ) )
				{
					cmsd_abort_proc( "エントリindexファイル'$checkpath'が書き込めません。ファイルのパーミッションが606（又は666）以上になっているかを確認してください。" );
				}
			}

			// エントリファイルが存在するならば、書き込み可能かをチェックする。
			$checkpath = App::createEntryFilePath( $this->entryName, $this->entryId );
			if ( file_exists( $checkpath ) )
			{
				if ( ! is_writable( $checkpath ) )
				{
					cmsd_abort_proc( "エントリindexファイル'$checkpath'が書き込めません。ファイルのパーミッションが606（又は666）以上になっているかを確認してください。" );
				}
			}
						
			// -- 処理開始 -----------------------------------------------------
			
			// 現在のメモリ上のXMLデータをDOMに読み込む。
			$xmlDocEntry = domxml_open_mem( $this->strAllXml );
			$xpathEntry = $xmlDocEntry->xpath_new_context();
			
			// indexデータをDOMに読み込む。
			$xmlDocEntryIndex = domxml_open_file( App::createEntryFilePath( $this->entryName, 'index' ) );
			$xpathEntryIndex = $xmlDocEntryIndex->xpath_new_context();
			$xmlNodeEntryIndexRoot = $xmlDocEntryIndex->document_element();
			
			// 新規エントリか？
			$bNewEntry = $this->isNewEntry(); //( $this->entryId == "new" );
			
			// 新規エントリの場合、新しいIDを付ける。
			if ( $bNewEntry )
			{
				$fp = fopen( App::createEntrySequenceFilePath( $this->entryName ), "ab+" );
				set_file_buffer($fp, 0);
				flock( $fp, LOCK_EX );
				
				// 新規IDを発行し、シーケンス管理ファイルに書き込む。
				$iSequence = 0;
				if ( file_exists( App::createEntrySequenceFilePath( $this->entryName ) ) )
				{
					//$fp = fopen( App::createEntrySequenceFilePath( $this->entryName ), "r" );
					rewind($fp);
					$iSequence = fgets($fp); //fread($fp, filesize( App::createEntrySequenceFilePath( $this->entryName ) ));
					//fclose($fp);
				}
				
				if ( is_numeric( $iSequence ) )
				{
					$iSequence = (int) $iSequence + 1;
					
				}
				else
				{
					$iSequence = 1;
				}
				
				if ( file_exists( App::createEntryFilePath( $this->entryName, sprintf( "%05d", $iSequence ) ) ) )
				{
					$this->errorMessage = "エントリIDの付与に失敗しました。エントリは保存されませんでした。$iSequence";
					flock( $fp, LOCK_UN );
					fclose( $fp );
					return;
				}
				
				ftruncate($fp, 0);	// ファイルサイズをゼロにする。
				rewind($fp);
				//$fp = fopen( App::createEntrySequenceFilePath( $this->entryName ), "w" );
				fwrite( $fp, $iSequence );
				
				flock( $fp, LOCK_UN );
				fclose($fp);
				
				$this->entryId = sprintf( "%05d", $iSequence );
				
			}
			else
			{
				// 既存エントリの場合、indexファイルから既存のエントリ情報を削除する。
				$xmlNodeNewEntryRefArr = $xpathEntryIndex->xpath_eval( "/entrylist/entry[@href='" . $this->entryName . '.' . $this->entryId . '.xml' . "']" );
				if ( cmsd_get_nodeset( $xmlNodeNewEntryRefArr ) )	// 編集中に他のユーザーに削除されている可能性がある為。
				{
					$xmlNodeNewEntryRef = $xmlNodeNewEntryRefArr->nodeset[0];
					$xmlNodeNewEntryRef->unlink_node();
				}
				
			}
			
			// エントリのヘッダ情報を書き込む。
			$xmlNodeEntry = $xmlDocEntry->document_element();
			$xmlNodeEntry->set_attribute( 'id', $this->entryId );
			$xmlNodeEntry->set_attribute( 'schemaname', $this->schemaName );
			/* 2004.12.23 エントリ更新日付をコンテンツマネージャから更新するようにした為、コメントアウト。
			$xmlNodeEntry->set_attribute( 'date', date( "Y-m-d H:i:s" ) );
			$xmlNodeEntry->set_attribute( 'year', date( "Y" ) );
			$xmlNodeEntry->set_attribute( 'month', date( "m" ) );
			$xmlNodeEntry->set_attribute( 'day', date( "d" ) );
			$xmlNodeEntry->set_attribute( 'hour', date( "H" ) );
			$xmlNodeEntry->set_attribute( 'minute', date( "i" ) );
			$xmlNodeEntry->set_attribute( 'second', date( "s" ) );
			*/
			
			//$xmlNodeEntry->set_attribute( 'visible', 'True' );
			$this->strAllXml = $xmlDocEntry->dump_mem( true, 'UTF-8');

			//$date = "2003-10-01 16:28:30"; 
			//sscanf( $date, "%d-%d-%d %d:%d:%d", &$yy, &$mm, &$dd, &$hh, &$ii, &$ss ); 
			//$mkdate = mktime( $hh, $ii, $ss, $mm, $dd 1, $yy); 
			//echo date("Y-m-d H:i:s", $mkdate);								
			
			// indexファイルにエントリを追加し、保存する。
			$xmlNodeNewEntryRef = $xmlDocEntryIndex->create_element( 'entry' );
			$xmlNodeNewEntryRef->set_attribute( 'href', $this->entryName . '.' . $this->entryId . '.xml' );
			$xmlNodeNewEntryRef->set_attribute( 'id', $this->entryId );
			
			// indexファイルにエントリの公開情報を保存する。
			$xmlNodeNewEntryRef->set_attribute( 'visible', $xmlNodeEntry->get_attribute('visible') );
			$xmlNodeNewEntryRef->set_attribute( 'startdate', $xmlNodeEntry->get_attribute('startdate') );
			$xmlNodeNewEntryRef->set_attribute( 'enddate', $xmlNodeEntry->get_attribute('enddate') );
			
			// indexファイルにその他のヘッダ情報を保存する。[2008.04 ユーザー権限]
			$xmlNodeNewEntryRef->set_attribute( 'date', $xmlNodeEntry->get_attribute('date') );
			$xmlNodeNewEntryRef->set_attribute( 'year', $xmlNodeEntry->get_attribute('year') );
			$xmlNodeNewEntryRef->set_attribute( 'month', $xmlNodeEntry->get_attribute('month') );
			$xmlNodeNewEntryRef->set_attribute( 'day', $xmlNodeEntry->get_attribute('day') );
			$xmlNodeNewEntryRef->set_attribute( 'hour', $xmlNodeEntry->get_attribute('minute') );
			$xmlNodeNewEntryRef->set_attribute( 'second', $xmlNodeEntry->get_attribute('second') );
			$xmlNodeNewEntryRef->set_attribute( 'weekday', $xmlNodeEntry->get_attribute('weekday') );
			
			$xmlNodeNewEntryRef->set_attribute( 'author', $xmlNodeEntry->get_attribute('author') );
			
			// indexエントリ情報に、グループキー、ソートキー情報を追加する。
			/// グループキー
			foreach( $this->groupKeys as $groupKey )
			{
				$xmlNodeGroupKeyDataArr = $xpathEntry->xpath_eval( '/entry/' . $groupKey ); // 現バージョンでは、トップ階層のみ対応。
				if ( !empty( $xmlNodeGroupKeyDataArr->nodeset ) )
				{
					$xmlNodeGroupKey = $xmlDocEntryIndex->create_element( 'groupkey' );
					$xmlNodeGroupKey->set_attribute( 'key', $groupKey );
					$xmlNodeGroupKey->set_content( $xmlNodeGroupKeyDataArr->nodeset[0]->get_content() );
					$xmlNodeNewEntryRef->append_child( $xmlNodeGroupKey );
				}
			}
			
			/// ソートキー
			$xmlNodeSortKeyDataArr = $xpathEntry->xpath_eval( '/entry/' . $this->sortKey ); // 現バージョンでは、トップ階層のみ対応。
			if ( !empty( $xmlNodeSortKeyDataArr->nodeset ) )
			{
				$sortValue = $xmlNodeSortKeyDataArr->nodeset[0]->get_content();
				$xmlNodeSortKey = $xmlDocEntryIndex->create_element( 'sortkey' );
				$xmlNodeSortKey->set_attribute( 'key', $this->sortKey );
				$xmlNodeSortKey->set_content( $sortValue );
				$xmlNodeNewEntryRef->append_child( $xmlNodeSortKey );
			}
			
			/// 日時
			$sortDate = $xmlNodeEntry->get_attribute('date');
			
			// ソートキーと日時により適切な位置を見つけ、indexにエントリ情報挿入する。
			$insertPos = -1; // 初期値
			$entryValue = ""; // 初期値 2011.04.14 追加
			$entryDate = ""; // 初期値 2016.10.22 追加
			$xmlNodeEntryRefArr = $xpathEntryIndex->xpath_eval('/entrylist/entry');
			foreach( cmsd_get_nodeset( $xmlNodeEntryRefArr ) as $index => $xmlNodeEntryRef )
			{
				$xmlNodeSortKeyArr = $xmlNodeEntryRef->get_elements_by_tagname( 'sortkey' );
				if ( ! empty( $xmlNodeSortKeyArr ) )
				{
					// エントリのソート値
					$entryValue = $xmlNodeSortKeyArr[0]->get_content(); // エントリのソート値
					$entryDate = $xmlNodeEntryRef->get_attribute( 'date' );
					
				}
				if ( $this->sortOrder === 'asc' )	// 昇順
				{
					if ( $entryValue == $sortValue )
					{
						// 値が同じなら、その中でエントリ日付が同じか過去のエントリを見つけてその位置を挿入位置とする
						if ( strcmp( $entryDate, $sortDate ) <= 0 )
						{
							$insertPos = $index;
							break;
						}
					}
					elseif ( $entryValue > $sortValue )
					{
						// 値が現在位置の方が大きいなら、エントリ日時は関係なく、その位置を挿入位置とする
						$insertPos = $index;
						break;
					}
				}
				else	// 降順
				{
					if ( $entryValue == $sortValue )
					{
						// 値が同じなら、その中でエントリ日付が同じか過去のエントリを見つけてその位置を挿入位置とする
						if ( strcmp( $entryDate, $sortDate ) <= 0 )
						{
							$insertPos = $index;
							break;
						}
					}
					elseif ( $entryValue < $sortValue )
					{
						// 値が現在位置の方が小さいなら、エントリ日時は関係なく、その位置を挿入位置とする
						$insertPos = $index;
						break;
					}
				
					/*
					if ( $entryValue < $sortValue && strcmp( $entryDate, $sortDate ) <= 0 )
					{
						$insertPos = $index;
						break;
					}
					*/
					
				}
			}
			
			if ( $insertPos <= -1 )
			{
				// 挿入位置が見つからなかった場合
				$xmlNodeEntryIndexRoot->append_child( $xmlNodeNewEntryRef );
				
				/*
				if ( $this->sortOrder == 'asc' )
				{
					// ascなら、最後尾に追加。
					$xmlNodeEntryIndexRoot->append_child( $xmlNodeNewEntryRef );
				}
				else
				{
					// descなら、先頭に追加。
					if ( !empty($xmlNodeEntryRefArr->nodeset) )
					{
						$xmlNodeEntryIndexRoot->insert_before( $xmlNodeNewEntryRef, $xmlNodeEntryRefArr->nodeset[0] );
					}
					else
					{
						$xmlNodeEntryIndexRoot->append_child( $xmlNodeNewEntryRef );
					}
				}
				*/
			}
			else
			{
				// 挿入位置が見つかった場合
				$xmlNodeEntryIndexRoot->insert_before( $xmlNodeNewEntryRef, $xmlNodeEntryRefArr->nodeset[$insertPos] );
				
			}
			
			//$xmlNodeEntryIndexRoot->append_child( $xmlNodeNewEntryRef );
			
			ADDom::SaveXmlToFile( $xmlDocEntryIndex, App::createEntryFilePath( $this->entryName, 'index' ), 'UTF-8' );
			//$xmlDocEntryIndex->dump_file( App::createEntryFilePath( $this->entryName, 'index' ), false, true );
			
						
			
			// アップロードファイルの処理。DOMツリー上の、更新された添付ファイルについて確定処理を行う。
			// 同時に、不要となった（参照されていない）ファイルを削除する。
			/// メモリ上のDOMツリーより、img/fileのノードを全て取得する。
			$xpathEntry = $xmlDocEntry->xpath_new_context();
			$xmlEntryImageArr = $xpathEntry->xpath_eval( "//*[@fileid!='']" );
			$attachedfiles = array();	// 添付ファイルのリストを作成する
			/* $smallimagefiles = array();	// 縮小画像ファイルのリストを別途作成する */
			if ( is_object($xmlEntryImageArr) ) {
				foreach( cmsd_get_nodeset( $xmlEntryImageArr ) as $xmlEntryImage )
				{
					$state = $xmlEntryImage->get_attribute('state');
					$fileName = $xmlEntryImage->get_attribute('src');
					
					if ( $state != 'saved' )
					{
						// ファイル名が空の場合は何もしない。
						if ( $fileName != '' )
						{
							// 既存ファイルが存在するなら、削除する。
							$filePath = App::createEntryDirPath( $this->entryName ) . $fileName;
							if ( file_exists( $filePath ) )
							{
								unlink( $filePath );
							}
							
							switch( $state )
							{
								case 'new' :
								case 'updated' :
								
									// tmpファイルをリネームする。
									if ( file_exists( $filePath . '.tmp' ) )
									{
										rename( $filePath . ".tmp", $filePath );
									}
									
									break;
								
								case 'deleted' :
									// 属性をクリアする。
									$xmlEntryImage->set_attribute( 'org', '' );
									$xmlEntryImage->set_attribute( 'filesize', '0' );
									$xmlEntryImage->set_attribute( 'src', '' );
									
									if ( $xmlEntryImage->get_attribute('filetype') === 'image' )
									{
										$xmlEntryImage->set_attribute( 'width', '0' );
										$xmlEntryImage->set_attribute( 'height', '0' );
									}
									
									// 画像パスをクリアする。2005.03.28追加
									EntryUtil::deleteTextNode( $xmlEntryImage );
									$xmlEntryImage->set_content('');

									break;
							
							}
							
							// entryidがnewの場合、ファイル名が未確定状態なので、正式な名前に変更する。
							if ( $bNewEntry )
							{
								$fileid = $xmlEntryImage->get_attribute('fileid');
								$org = $xmlEntryImage->get_attribute('org');
								$oldFileName = $fileName;
								$fileName = $this->getSrcFileName( $fileid, $org );
								$xmlEntryImage->set_attribute('src', $fileName );
								
								if ( $xmlEntryImage->get_attribute('filetype') === 'image' )
								{
									$fileurl = App::createImageUrl( $this->entryName, $this->entryId, $fileid, $org );	
								}
								else
								{
									$fileurl = App::createFileUrl( $this->entryName, $this->entryId, $fileid, $org );	
								}
								
								// ファイルのURLを内容としてセット。
								EntryUtil::deleteTextNode( $xmlEntryImage );
								$xmlEntryImage->set_content( fileconv($fileurl) );
								
								// テンポラリファイルを正式名にリネーム
								if ( file_exists( App::createEntryDirPath( $this->entryName ) . $oldFileName ) )
								{
									rename( App::createEntryDirPath( $this->entryName ) . $oldFileName, App::createEntryDirPath( $this->entryName ) . $fileName );
								}
							}
						}
						
						// stateをsavedに変更する。
						$xmlEntryImage->set_attribute('state', 'saved');
					}
					
					// 添付ファイル名をリストに追加。
					$attachedfiles[] = $fileName;
					
					/*
					// 縮小画像ファイル名をリストに追加。
					if ( strlen( $xmlEntryImage->get_attribute('smallimage') ) > 0 )
					{
						$fileid = $xmlEntryImage->get_attribute('fileid');
						$org = $xmlEntryImage->get_attribute('org');
						$smallIMGFilename = $this->getSrcFileName( $fileid . '.small', $org );
						$smallimagefiles[] = $smallIMGFilename;
					}
					*/
				}
			}
			
			
			/// 不要となったファイルを削除する。(このエントリ内の *.tmpファイルもここで消える）
			$filesInEntry = glob( App::createEntryDirPath( $this->entryName) . $this->entryName . '.' . $this->entryId . '.*' );
			if ( $filesInEntry )
			{
				foreach ( $filesInEntry as $filePath )
				{
					// エントリ内の画像ファイルリストに含まれていなければ、不要とみなして削除する。
					if ( ! in_array( basename( $filePath ), $attachedfiles ) ) // and ! in_array( basename( $filePath ), $smallimagefiles ) )
					{
						if ( getfileext( basename( $filePath ) ) != 'xml' )
						{
							//print basename( $filePath ) . " is deleted(img).<br>\n"; 
							unlink( $filePath );
						}
					}
				}
			}
			
			/// このエントリ内の画像キャッシュファイルを全て削除する。（本来、変更されたもの＆削除されたものだけでOKだが、処理簡素化の為に全削除する）
			$cacheImageFilesInEntry = glob( App::getCacheDir() . "imagecache.{$this->entryName}.{$this->entryId}.*" );
			if ( $cacheImageFilesInEntry )
			{
				foreach ( $cacheImageFilesInEntry as $filePath )
				{
					unlink( $filePath );
				}
			}
			
			
			/// ゴミのtmpファイルを全て削除する。（過去に、編集開始した後保存せずにブラウザを閉じた場合のゴミなど）
			$tempfilesInEntry = glob( App::createEntryDirPath( $this->entryName) . '*.tmp' );
			if ( $tempfilesInEntry )
			{
				foreach ( $tempfilesInEntry as $tempfilePath )
				{
					// 1日以上前のファイルだったら削除する（現在のファイルを消してしまうと、作業中の他のユーザーに影響してしまう為）
					$timeLastUpdate = filemtime( $tempfilePath );
					if ( $timeLastUpdate < strtotime( '-1 day' ) )
					{
						unlink( $tempfilePath );
					}
				}
			}
			
			// XMLデータをメモリに戻す。
			$this->strAllXml = $xmlDocEntry->dump_mem( true, 'UTF-8' );
			
			// DOMを書き込む。
			$xmlDocEntry->dump_file( App::createEntryFilePath( $this->entryName, $this->entryId ), false, true );
			
			
			// 全キャッシュファイルの削除 2009.01.30 追加 j.shinde
			ADPageCache::deleteAllCaches();
			
			// メッセージを表示する。
			$this->normalMessage = 'エントリを保存しました。';
			
		}
		
		
		function delete()
		{
			
		}
		
		function getSrcFileName( $fileid, $orgFileName )
		{
			return $this->entryName . '.' . $this->entryId . '.' . $fileid . '.' . getfileext( $orgFileName );
		}
		
		function getUpperLevelXPathStr( $strXPath )
		{
			// 現在のxpathから、1階層("/"で区切られた部分）分、削除する。
			$pos = strrpos( $strXPath, '/' );
			$strXPath = substr( $strXPath, 0, $pos );
			
			return $strXPath;

		}
		

		/* ----------------------------------------------------------------
		 * FORM-XMLを生成する。
		 */
		function renderFormXML()
		{
			// Site.Configより指定されたエントリ設定情報を取得する。
			/*
			$xmlDocSite = domxml_open_file( App::getSiteConfigFilePath() );
			$xpathSite = $xmlDocSite->xpath_new_context();
			$xmlEntryConfigList = $xpathSite->xpath_eval( "/site/entries/entry[@name='" . $this->entryName . "']" );
			$entryCaption = $xmlEntryConfigList->nodeset[0]->get_attribute('caption');
			*/
			$siteConfig = new CMSSiteConfig();
			$entryConfig = $siteConfig->getEntryConfig( $this->entryName );
			$entryCaption = $entryConfig->caption;
			
			// スキーマのXpathを生成する。
			$xmlDocSchema = domxml_open_file( App::createSchemaFilePath( $this->schemaName ) );
			//print "XML string : '<pre>" . htmlentities( htmlconv( $xmlDocSchema->dump_mem() ), ENT_COMPAT, 'EUC-JP' ) . "</pre>'";
			$xpathSchema = $xmlDocSchema->xpath_new_context();
			
			// 現在位置のスキーマを読み込む。
			$xmlSchemaRoot = $xpathSchema->xpath_eval( $this->currentSchema ); // "/schema/data[@name='shopphotolist']" );
			$xmlSchemaRoot = $xmlSchemaRoot->nodeset[0];
		
			// エントリのXPathを生成する。			 
			//print "XML string : '<pre>" . htmlentities( htmlconv( $this->strAllXml ), ENT_COMPAT, 'EUC-JP' ) . "</pre>'"; 		
			$xmlDocEntry = domxml_open_mem( $this->strAllXml );
			$xpathEntry = $xmlDocEntry->xpath_new_context();

			// HTML生成開始	
			// DOMをを生成。
			// ルートノード'entryeditor'を生成。
			$xmlDocOutput = domxml_new_doc( '1.0' );
			$xmlOutputRoot = $xmlDocOutput->add_root( 'entryeditor' );	
			
			// Entry情報の出力
			$xmlOutputEntry = $xmlDocOutput->create_element('entry');
			$xmlOutputEntry->set_attribute('id', $this->entryId );
			$xmlOutputEntry->set_attribute('isnew', ( $this->isNewEntry() ? 'True' : 'False' ) );
			$xmlOutputEntry->set_attribute('isdirty', ( $this->isDirty ? 'True' : 'False' ) );
			$xmlOutputEntry->set_attribute('entrycaption', fileconv( $entryCaption ) );
			$xmlOutputEntry->set_attribute('entryname', $this->entryName );
			/// 現在位置のスキーマ・エントリに基づいて編集フォーム項目を生成する。
			if ( $xmlSchemaRoot->get_attribute( "type" ) === 'list' )
			{
				// type=="list"の場合、リスト用のXMLを生成する。
				$this->renderFormXMLList( $xpathSchema, $xpathEntry, $xmlOutputEntry, $entryConfig );
			}
			else
			{
				// type=="list"以外の場合、ブロック用のXMLを生成する。トップ階層の場合もtypeがnullになるのでここへ入る。
				$this->renderFormXMLBlock( $xmlSchemaRoot->child_nodes(), $xpathEntry, $xmlOutputEntry );
			}			
			$xmlOutputRoot->append_child( $xmlOutputEntry );
			
			// ReloadTokenの出力
			ReloadToken::embedToXmlDoc( $xmlDocOutput );
			/*
			$xmlOutputReloadToken = $xmlDocOutput->create_element('reloadtoken');
			$xmlOutputReloadToken->set_attribute('key', ReloadToken::getTokenKey() );
			$xmlOutputReloadToken->set_attribute('value', ReloadToken::getCurrentToken() );
			$xmlOutputRoot->append_child( $xmlOutputReloadToken );
			*/
			
			// エントリ内位置情報の出力
			$xmlOutputLocation = $xmlDocOutput->create_element('location');
			$xmlOutputLocation->set_attribute('istop', ( ( $this->currentEntry === '/entry' ) ? 'True' : 'False' ));
			$xmlOutputLocation->set_attribute('currenttype', ( $xmlSchemaRoot->get_attribute( "type" ) === 'list' ? 'list' : 'block' ));
			$this->createOutputSchemaLocation( $xpathSchema, $xpathEntry, $xmlOutputLocation );
			$xmlOutputRoot->append_child( $xmlOutputLocation );
			
			// エラーIDの出力
			if ( $this->errorId != '' )
			{
				$xmlOutputRoot->set_attribute('errorid', $this->errorId );
			}
			
			// エラーメッセージの出力
			if ( $this->errorMessage != '' )
			{
				$xmlOutputErrorMessage = $xmlDocOutput->create_element('errormessage');
				$xmlOutputErrorMessage->set_content( fileconv( $this->errorMessage ) );
				$xmlOutputRoot->append_child( $xmlOutputErrorMessage );
			}
			
			// 通常メッセージの出力
			if ( $this->normalMessage != '' )
			{
				$xmlOutputNormalMessage = $xmlDocOutput->create_element('normalmessage');
				$xmlOutputNormalMessage->set_content( fileconv( $this->normalMessage ) );
				$xmlOutputRoot->append_child( $xmlOutputNormalMessage );
			}
			
			// エントリ一件表示用URLが指定されていたら、出力する。
			if ( ! empty($entryConfig->entryPageUrl) )
			{
				$xmlOutputRoot->set_attribute('entrypageurl', $entryConfig->entryPageUrl );
			}
			
			// ログイン情報の出力
			$authedUser = LoginManager::GetAuthedUser();
			$xmlOutputAuthedUser = $xmlDocOutput->create_element( 'autheduser' );
			$xmlOutputAuthedUser->set_attribute( 'userid', $authedUser->getUserid() );
			$xmlOutputAuthedUser->set_attribute( 'role', $authedUser->getRole() );
			$xmlOutputRoot->append_child( $xmlOutputAuthedUser );
			
			// 全ユーザー情報の出力（Adminの場合のみ）
			if ( $authedUser->isAdministrator() )
			{
				$userArr = SiteUser::LoadAllUsers();
				$xmlOutputUserList = $xmlDocOutput->create_element( 'userlist' );
				foreach( $userArr as $user )
				{
					$xmlOutputUser = $xmlDocOutput->create_element( 'user' );
					$xmlOutputUser->set_attribute( 'userid', $user->getUserId() );
					$xmlOutputUserList->append_child( $xmlOutputUser );
				}
				$xmlOutputRoot->append_child(  $xmlOutputUserList );
			}
			
			// ページ戻り先情報の出力
			$xmlOutputOwnerPage = $xmlDocOutput->create_element( 'ownerpage' );
			$xmlOutputOwnerPage->set_attribute( 'name', 'entries' ); // 現状使っていない情報
			$xmlOutputOwnerPage->set_attribute( 'maxpagerows', $this->entries_MaxPageRows );
			$xmlOutputOwnerPage->set_attribute( 'pageno', $this->entries_PageNo );
			$xmlOutputRoot->append_child( $xmlOutputOwnerPage );
			
			// HTML生成終了
			// XSL変換処理
			//$strFormHTML = ADXslt::process( $xmlDocOutput, dirname(__FILE__) . "/entryeditor.xsl" );
			$strFormHTML = CommonActionUtil::XsltEntryEditor( $xmlDocOutput );
			/*
			$xslt = domxml_xslt_stylesheet_file( dirname(__FILE__) . "/entryeditor.xsl" );
			$result = $xslt->process( $xmlDocOutput );
			$strFormHTML = $result->dump_mem( false, 'EUC-JP' );			
			*/
			
			//$strFormHTML = "XML string : <pre>" . htmlconv( $xmlDocOutput->dump_mem( true, 'EUC-JP' ) ) . "<pre>'";
			
			// 文字化け不具合回避コード。2006.07.06 追加
			$strFormHTML = replaceMetaTag( $strFormHTML, 'utf-8', gCMSDInternalEncoding() );
			
			return $strFormHTML;
		}
		
		/* ----------------------------------------------------------------
		 * FORM-HTMLを生成する（内部処理）。
		 * $nodeList : 現在位置のスキーマの"data"要素のリスト。
		 * $xpathEntry : メモリ中にある、編集中のエントリデータ
		 * $xmlOutputEntry : 編集画面表示用のデータ。ルート要素。
		 */
		function renderFormXMLBlock( $nodeList, $xpathEntry, &$xmlOutputEntry )
		{
			// トップ階層の場合、エントリ情報（現在は更新日付のみ）の入力欄を表示する。
			if ( $this->currentEntry === '/entry' )
			{
				// 画面表示用XMLのドキュメントを取得。
				$xmlDocOutput = $xmlOutputEntry->owner_document();
				
				// エントリ更新日時を、通常の日時項目クラスのレンダリングを使って出力する。----------------
				
				/// - エントリ更新日時表示用の項目を生成。（type='entrydate'で生成している）
				$xmlSchemaDataEntryDate = $this->createSchemaDataForEntryDate( $xmlDocOutput );
				$xmlDocOutput = $xmlOutputEntry->owner_document();
				$xmlOutputEntryData = $xmlDocOutput->create_element('entrydata');
				$xmlOutputEntryData->set_attribute('name', $xmlSchemaDataEntryDate->get_attribute( "name" ) );
				$xmlOutputEntryData->set_attribute('caption', $xmlSchemaDataEntryDate->get_attribute( "caption" ) );
				$xmlOutputEntryData->set_attribute('type', $xmlSchemaDataEntryDate->get_attribute('type') );
				
				/// - 日時項目オブジェクトを生成してOutputへレンダリング
				$item = EntryItemFactory::createEntryItem( $xmlOutputEntryData, $this->entryName, $this->entryId	);
				$item->renderFormItemXML( $xpathEntry, $this->currentEntry, $xmlOutputEntryData );
				$xmlOutputEntry->append_child( $xmlOutputEntryData );
				
				// Configの読み込み
				$siteConfig = new CMSSiteConfig();
				$entryConfig = $siteConfig->getEntryConfig( $this->entryName );
				
				// エントリ更新日時以外のヘッダ情報（公開情報など）をレンダリングする。---------------
				$item = new HeaderEntryItem( $this->entryName, $this->entryId, $entryConfig->publishControl );
				$xmlOutputEntryData = $xmlDocOutput->create_element('entrydata');
				$xmlOutputEntryData->set_attribute('name', 'header' ); // ダミー。この値は実際には使用されない。
				$xmlOutputEntryData->set_attribute('caption', fileconv( '公開設定' ) );
				$xmlOutputEntryData->set_attribute('type', 'header' );
				
				$item->renderFormItemXML( $xpathEntry, $this->currentEntry, $xmlOutputEntryData );
				$xmlOutputEntry->append_child( $xmlOutputEntryData );
				

			}
			
			foreach( $nodeList as $node )
			{
				if ( $node->node_type() == XML_ELEMENT_NODE )
				{
					// スキーマから各値を取得して出力
					$xmlDocOutput = $xmlOutputEntry->owner_document();
					$xmlOutputEntryData = $xmlDocOutput->create_element('entrydata');
					$xmlOutputEntryData->set_attribute('name', $node->get_attribute( "name" ) );
					$xmlOutputEntryData->set_attribute('caption', $node->get_attribute( "caption" ) );
					$xmlOutputEntryData->set_attribute('description', $node->get_attribute( 'description' ) );	// 2006.2.8 セパレータ項目追加対応
					$xmlOutputEntryData->set_attribute('type', $node->get_attribute('type') );

					$item = EntryItemFactory::createEntryItem( $node, $this->entryName, $this->entryId	);
					$item->renderFormItemXML( $xpathEntry, $this->currentEntry, $xmlOutputEntryData );
					
					$xmlOutputEntry->append_child( $xmlOutputEntryData );
				}
			}
			
		}
		
		function renderFormXMLList( $xpathSchema, $xpathEntry, &$xmlOutputEntry, $entryConfig )
		{
			// スキーマの現在位置の子要素（つまり繰り返し要素のスキーマ定義）を取得する。
			$xmlSchemaListItemArr = $xpathSchema->xpath_eval( $this->currentSchema . "/listitem" );
			$xmlSchemaListItem = $xmlSchemaListItemArr->nodeset[0];	// listitem要素は1つしか有りえない
			
			// 見出し用の項目名を取得
			$titleItemNames = explode( ",", $xmlSchemaListItem->get_attribute('title') );
			
			if ( ! is_array( $titleItemNames ) )
			{
				// カンマ区切りで複数指定されてなければ単純値が返る為、ここで1要素の配列にする。
				$titleItemNames = array( $titleItemNames );
			}
			
			$titleItemNamesArr = array();
			$titleItemTypesArr = array();
			
			if ( is_array( $titleItemNames )  )
			{
				// 結果が配列の場合、trimして空白でなければ見出し項目名配列に格納
				foreach( $titleItemNames as $titleItemName )
				{
					$titleItemName = trim( $titleItemName );
					if ( $titleItemName !== '' )
					{
						$xmlTitleItemArr = $xpathSchema->xpath_eval( $this->currentSchema . "/listitem/data[@name='{$titleItemName}']" );
						if ( count( cmsd_get_nodeset( $xmlTitleItemArr ) ) > 0 )	// その項目が存在したら。
						{
							$xmlTitleItem = $xmlTitleItemArr->nodeset[0];
							$titleItemNamesArr[] = $xmlTitleItem->get_attribute('name');
							$titleItemTypesArr[] = $xmlTitleItem->get_attribute('type');
						}
					}
				}
			}
			
			// 見出し項目が取得できなければ、先頭項目を見出しとする。
			if ( count( $titleItemNamesArr ) <= 0 )
			{
				$xmlSchemaListItemFirstChildArr = $xpathSchema->xpath_eval( $this->currentSchema . "/listitem/data[@type!='separator'][1]" );	// 2006.11.14 変更 セパレータ項目追加対応（対応遅れ）
				$xmlSchemaListItemFirstChild = $xmlSchemaListItemFirstChildArr->nodeset[0];	// リスト項目内の最初の子スキーマ項目
				$titleItemNamesArr[0] = $xmlSchemaListItemFirstChild->get_attribute('name');
				$titleItemTypesArr[0] = $xmlSchemaListItemFirstChild->get_attribute('type');
			}
			
			
			// 繰り返し要素スキーマの説明文を取得する。
			$datacaption = convencoding_xml2html( $xmlSchemaListItem->get_attribute( 'caption' ) );
			
			// エントリの現在位置の子要素（つまり繰り返し要素のリスト）を取得する。
			//$itemNodeName = $xmlSchemaNode->child_nodes()->nodeset[0]->get_attribute( 'name' );
			$xmlEntryNodeList = $xpathEntry->xpath_eval( $this->currentEntry . "/*" ); // $itemNodeName" );
			
			$xmlDocOutput = $xmlOutputEntry->owner_document();
			
			// エントリリスト項目の子ノード毎に、ノードの情報を１行ずつ出力する。
			foreach( cmsd_get_nodeset( $xmlEntryNodeList ) as $nodeChild )
			{
				if ( $nodeChild->node_type() == XML_ELEMENT_NODE )
				{
					$listitemId = $nodeChild->get_attribute( 'id' );
					
					// リスト見出しを取得する（複数の場合は"｜"で連結する）。
					$caption = '';	// 初期値
					
					foreach( $titleItemNamesArr as $index=>$titleItemName )
					{
						if ( strlen( $caption ) > 0 ) { $caption .= '｜'; }
					
						$xmlTitleItemArr = $xpathEntry->xpath_eval( $this->currentEntry . "/listitem[@id='{$listitemId}']/{$titleItemName}" );
						if ( count( cmsd_get_nodeset( $xmlTitleItemArr ) ) <= 0 )
						{
							// その項目が存在しなかったら、固定値を返す。
							$caption .= "(未設定)";
							continue;	// 次の見出しへ。
						}
						
						$xmlTitleItem = $xmlTitleItemArr->nodeset[0];
						
						switch (  $titleItemTypesArr[$index] )
						{
							// ファイルの場合、ファイル名を表示
							case 'img':
							case 'file':
								// img項目かfile項目の場合、内容がファイルのURLになっていて長すぎるので、オリジナルファイル名を見出しとする。
								$caption .= file2code( $xmlTitleItem->get_attribute( 'org' ) );
								break;
							
							// メニューの場合、選択肢キャプションを表示
							case 'menu':
								$menuId = $xmlTitleItem->get_content();
								
								// メニュー項目が外部データを参照しているかどうか？（2015/1/20対か）
								$dataSource = $entryConfig->getDatasource( $titleItemName );
								if (is_null($dataSource)) {
									// 外部データ無し。スキーマからメニュー項目ラベルを取得する。
									$xmlSchemaMenuItemArr = $xpathSchema->xpath_eval( $this->currentSchema . "/listitem/data[@name='{$titleItemName}']/menuitem[@id='{$menuId}']" );
									
									if ( !empty($xmlSchemaMenuItemArr->nodeset) )
									{
										$caption .= file2code( $xmlSchemaMenuItemArr->nodeset[0]->get_content() );
									}
									else
									{
										$caption .= "(不明[id:{$menuId}])";
									}
									
								}
								else {
									// 外部データ有り。データソースからメニュー項目ラベルを取得する。
									if ( isset( $dataSource->menuItems[$menuId] ) ) {
										$caption .= $dataSource->menuItems[$menuId];
									}
									else
									{
										$caption .= "(不明[id:{$menuId}])";
									}
								}
								
								break;
							
							// それ以外は内容をそのまま
							default:
								// それ以外の場合は、内容を見出しとする。
								$caption .= file2code( $xmlTitleItem->get_content() );
								break;
						}
						
					}
					
					$caption = mb_substr( $caption, 0, 50 );	// 先頭50文字でカットする。。
					
					// 子要素のIDを出力する。	
					$xmlOutputEntryData = $xmlDocOutput->create_element('entrydata');
					$xmlOutputEntryData->set_attribute('caption', fileconv($caption) );
					$xmlOutputEntryData->set_attribute('listid', $nodeChild->get_attribute( "id" ) );
					
					// 項目の選択状態を保持する。
					if ( is_array( $this->selectedListItems ) and in_array( $nodeChild->get_attribute('id'), $this->selectedListItems ) )
					{
						$xmlOutputEntryData->set_attribute('selected', 'selected' );
					}
					
					$xmlOutputEntry->append_child( $xmlOutputEntryData );
				}
			}

		}
		
		/* ----------------------------------------------------------------
		 * 現在のエントリ階層のパンくずナビゲーションHTMLを生成する。
		 * 
		 */
		function createOutputSchemaLocation( $xpathSchema, $xpathEntry, &$xmlOutputLocation )
		{
			
			$schemaPath = $this->currentSchema;
			$entryPath = $this->currentEntry;
			
			// 階層数の取得
			$iMaxLevel = count( explode( '/', $schemaPath ) ) - 2;
			
			$iLevel = $iMaxLevel;  // 最下層からスタート（リンクURL作成に用いる）
			
			$xmlDocOutput = $xmlOutputLocation->owner_document();
			$xmlOutputLocation->set_attribute('maxlevel', $iMaxLevel );
			
			while ( $schemaPath != "/schema" )
			{
				// エントリからリストIDを取得する（リストじゃない場合は空が入る）
				//print $entryPath;
				$xmlEntryArr = $xpathEntry->xpath_eval( $entryPath );
				$xmlEntry = $xmlEntryArr->nodeset[0];
				$listID = $xmlEntry->get_attribute('id');
				
				// スキーマからキャプションを取得する。
				$xmlSchemaArr = $xpathSchema->xpath_eval( $schemaPath );
				$xmlSchema = $xmlSchemaArr->nodeset[0];
				$schemaCaption = $xmlSchema->get_attribute('caption');
				
				if ( !empty( $listID ) )
				{
					$schemaCaption .= '(' . $listID . ')';
				}
				
				$xmlOutputLocationItem = $xmlDocOutput->create_element('locationitem');
				$xmlOutputLocationItem->set_attribute('distance', $iMaxLevel - $iLevel );
				$xmlOutputLocationItem->set_attribute('level', $iLevel );
				$xmlOutputLocationItem->set_attribute('caption', $schemaCaption );
				$xmlOutputLocation->append_child( $xmlOutputLocationItem );
				
				// XPathの階層を一つ上に上がる。
				$schemaPath = $this->getUpperLevelXPathStr( $schemaPath );
				$entryPath = $this->getUpperLevelXPathStr( $entryPath );
				
				$iLevel --;
			}
			
		}
		
		/* ----------------------------------------------------------------
		 * エントリ更新日付を編集する為のスキーマ定義オブジェクトを生成する。
		 * 
		 */		
		function createSchemaDataForEntryDate( &$xmlDocSchema )
		{
			$xmlSchemaDataEntryDate = $xmlDocSchema->create_element( 'data' );
			$xmlSchemaDataEntryDate->set_attribute('name', 'cmsd_date');
			$xmlSchemaDataEntryDate->set_attribute('type', 'entrydate');
			$xmlSchemaDataEntryDate->set_attribute('caption', fileconv( 'エントリ更新日時' ));
			
			return $xmlSchemaDataEntryDate;
				
		}
			
	}
	
	


?>